%------------------------------------------------------------------------------
% Routine          : set15d()
%
% Description      : place elements and points on surface of 1.5 D array.
%		     surface is rectangular or oval
%		     Array may be planar or curved and/or have fixed focus
%                    in azimuth/elevation.
%		     1.5D Array : The number of points on each element in
%		     elevation is proportional to the aperture of the element.
%		     The points on each element are placed according to 
%		     a uniform grid
%
%		     Mm = 3 | 5: 1.5 D array without electronic focus (centers(2,:) = 0)
% Unimplemented idea:
%		     Mm = -3 | -5: 1.5 D array with electronic focus (centers(2,:) = 
%                                                           center of elevation element)
%			
%		     
% Output data structure:
%
% 		     el_pts(1,n) = x
% 		     el_pts(2,n) = y
% 		     el_pts(3,n) = z 
% 		     el_pts(4,n) = nn % element number in major dimension (azimuth)
% 		     el_pts(5,n) = mm % element number in minor dimension (elevation)
%
%
% Language         : Matlab syntax
%
% Version no.      : 1.0 Kari Lervik, September 1992
% Version no.      : 1.1 SH  20.03.93 completely rewritten 
% Version no.      : 1.2 SH  12.05.93 updated
% Version no.      : 1.3 TKL 14.07.93 put fid=1 in file for unifying printouts
% Version no.	   : 1.4 FT  27.07.93 1.5D Array
% Version no.	   : 1.5 TKL 21.10.93 Corrected elevation point pitch 
%                  : 1.6 SH   3.12.93 Also corrected elevation point pitch!
%                                     Added routines for finding fit to el. lens
%		     2.0 SH  31.05.96 Separated from rectangu.m to set15d.m
%				      Added output of centers
%
% Called by        : rectangu.m
% Routines called  : in_out.m, circfit.m
% Load files       :
%
%------------------------------------------------------------------------------
	function [el_pts, centers] = set15d(transducer,excitation,media);

% VEB 17/3-93
a=transducer(5);
ai=transducer(12); 
am=transducer(13); 
ao=transducer(14); 
Mm=transducer(6);
Nn=transducer(2);
om=transducer(15);
oo=transducer(16);
ri=transducer(9);
rm=transducer(10);
ro=transducer(11);
d=transducer(1);
% SH 19.3.93:
mm = transducer(7);
nn = transducer(3);
ROC = transducer(17);
shape = transducer(18);

npts = Nn*nn*mm;   %total number of points in
						  %1D rectangular array

if (shape >= 0 | shape == -8),

    centers = zeros(5,Nn*Mm);


% VEB 17/3-93

	if Mm == 3, 
%     	-----------------------generate coordinates, start at origin
%	FT 19. July 1993
%	1.5D ARRAY 	3 elements in elevation dimension
% 

    
    d_az = d/Nn;

    % Determine number of points on each elevation element

    mmi = round(mm*ai/(ai + 2*ao));
    mmo = round(mm*ao/(ai + 2*ao));

    if (mmi >0)
    	disp(['Number of points, Inner Elevation Element : ',num2str(mmi)]);
    else
	disp('       WARNING!!!  No points in Inner Elevation Element!')
    end
    if (mmo>0)
    	disp(['Number of points, Outer Elevation Element : ',num2str(mmo),' * 2']);
    else
	disp('       WARNING!!!  No points in Outer Elevation Element!')
    end

    npts = Nn*nn*(mmi + 2*mmo);		%total number of points in
					%1.5D rectangular array

% allocating memory

    if (shape == 0) 
	el_pts = zeros(5,npts-10);
    else
	el_pts = zeros(5,round(npts/3*2));
    end;    



    if (ROC == 0 | abs(ROC) == Inf),

% rectangular array:

	n = 0;
	m = 1;
	for az = 0:Nn-1,
		x_center = d_az*(-Nn/2 + 0.5 + az);

		temp = [x_center, 0, 0, az+1];
		centers(1:5,m:m+2) = [temp ,1; temp, 2; temp, 3]';
		m=m+3;

		for az_pt = 0:nn-1,
			x_delta =  (d_az/nn)*(-nn/2 + 0.5 + az_pt);
			xx = x_center + x_delta;
			[yy, in] = in_out(xx, d/2, a/2, shape);
		   for el = -1:1,
			if (el == 0), 			% inner elevation element
				mel = mmi;
				d_el = ai;
				rad = ri;
			else				% outer elevation element
				mel = mmo;
				d_el = ao;
				rad = ro;
			end	%ifel=0
			y_center = el*(ai+ao)/2;    	% center of elevation element

			for el_pt = 0:mel-1,
				y_delta = y_center + (d_el/mel)*(-mel/2 + 0.5 + el_pt);
%
% check that point is inside aperture contour
%
				[yy, in] = in_out(xx, d/2, a/2, shape);
				if (in == 1 & abs(y_delta) <= yy),
					n = n + 1;
					el_pts(1,n) = x_center + x_delta;
					el_pts(2,n) = y_delta;
					if (rad ~= 0 & finite(rad)),
					   el_pts(3,n) = abs(el)*oo + rad - sign(rad)* ... 
							 sqrt(rad^2 - y_delta^2);
					else
					   el_pts(3,n) = abs(el)*oo;
 					end	%ifrad~=0
					el_pts(4,n) = az + 1;
					el_pts(5,n) = el + 2;
				end	%ifin==1
			end	%forel_pt
		    end	%forel
		end	%foraz_pt
	end	%foraz
     else  %if (ROC == 0

% curved array
% ROC > 0 => fixed focus in azimuth
% ROC < 0 => curved array

	n = 0;
	m = 1;
	
	theta_open = 2*asin(d/(2*ROC));
	disp(['Azimuth angle: ',num2str((180/pi)*theta_open),' degrees']);
	d_theta = abs(theta_open)/Nn;

	for az = 0:Nn-1,

		theta_center = d_theta*(-Nn/2 + 0.5 + az);

		temp = [abs(ROC)*sin(theta_center), 0,ROC*(1-cos(theta_center)), az+1];
		centers(1:5,m:m+2) = [temp, 1; temp, 2; temp, 3]';
		m = m+3;

 		for az_pt = 0:nn-1,
			theta_delta =  (d_theta/nn)*(-nn/2 + 0.5 + az_pt);

		    for el = -1:1
			if (el == 0),			%inner elevation element
				mel = mmi;
				d_el = ai;
				rad = ri;
			else				%outer elevation element
				mel = mmo;
				d_el = ao;
				rad = ro;
			end	%ifel=0
			y_center = el*(ai+ao)/2;        %center of  elevation element

			for el_pt = 0:mel-1,
				y_delta = y_center + (d_el/mel)*(-mel/2 + 0.5 + el_pt);
				thet = theta_center + theta_delta;
%
% elevation focusing correction:
%				  
				if (rad ~= 0 & abs(rad) ~= Inf),
				    temp = rad^2 - y_delta^2;
				    if (temp > 0)
					delta_r = abs(el)*oo + rad - sign(rad)*sqrt(temp);
				    else
				  	delta_r = abs(el)*oo + rad;
				    end  %if (temp > 0)
				else  %if (rad ~= 0
					delta_r = abs(el)*oo;
				end  %if (rad~= 0
 				xx = (abs(ROC) + delta_r)*sin(thet);
			 				  
%
% check that point is inside aperture contour
%
				[yy, in] = in_out(xx, d/2, a/2, shape);
				if (in == 1 & abs(y_delta) <= yy),
					n = n + 1;
					el_pts(1,n) = xx;
					el_pts(2,n) = y_delta;
	 				el_pts(3,n) = sign(ROC)*(abs(ROC) - sqrt(ROC^2 - xx^2)) + ...
                                              delta_r*cos(thet);
					el_pts(4,n) = az + 1;
					el_pts(5,n) = el + 2;
				end  %if (in == 1
			end  %for el_pt
		    end  %for el
		end  %for az_pt
	end  %for az 

    end  %%if (ROC == 0
   	coglob = 'y';
	elseif Mm == 5,  %elseif Mm == 3    
%     	-----------------------generate coordinates, start at origin
%	FT, 19 July 1993 :
%	1.5D ARRAY 	5 elements in elevation dimension
% 

    d_az = d/Nn;

    % Determine number of points on each elevation element

    mmi = round(mm*ai/(ai + 2*am + 2*ao));
    mmm = round(mm*am/(ai + 2*am + 2*ao));
    mmo = round(mm*ao/(ai + 2*am + 2*ao));
    if (mmi>0)
    	disp(['Number of points, Inner Elevation Element : ',num2str(mmi)]);
    else
	disp('       WARNING!!!  No points in Inner Elevation Element!')
    end
    if (mmm>0)
    	disp(['Number of points, Middle Elevation Element : ',num2str(mmm),' * 2']);
    else
	disp('       WARNING!!!  No points in Middle Elevation Element!')
    end
    if (mmo>0)
   	disp(['Number of points, Outer Elevation Element : ',num2str(mmo),' * 2']);
    else
	disp('       WARNING!!!  No points in Outer Elevation Element!')
    end

    npts = Nn*nn*(mmi + 2*mmo + 2*mmm);		%total number of points in
						%1.5D rectangular array

% allocating memory

    if (shape == 0) 
	el_pts = zeros(5,npts-10);
    else
	el_pts = zeros(5,round(npts/3*2));
    end;

    if (ROC == 0 | abs(ROC) == Inf),

% rectangular array:

	n = 0;
	m = 1;
	for az = 0:Nn-1,
		x_center = d_az*(-Nn/2 + 0.5 + az);

		temp = [x_center, 0, 0, az+1];
		centers(1:5,m:m+4) = [temp,1; temp,2; temp,3; temp,4; temp,5]';
		m=m+5;

		for az_pt = 0:nn-1,
			x_delta =  (d_az/nn)*(-nn/2 + 0.5 + az_pt);
			xx = x_center + x_delta;
			[yy, in] = in_out(xx, d/2, a/2, shape);
		   for el = -2:2,
			if (el == 0),			% inner elevation element
				mel = mmi;
				d_el = ai;
				rad = ri;
			elseif(abs(el) == 1),		% middle elevation element
				mel = mmm;
				d_el = am;
				rad = rm;
			else				% outer elevation element
				mel = mmo;
				d_el = ao;
				rad = ro;
			end	%ifel=0
			y_center = el*(ai+am)/2 + el*(abs(el)-1)*(ao-ai)/4; %center of
									    % elevation element
			for el_pt = 0:mel-1,
				y_delta = y_center + (d_el/mel)*(-mel/2 + 0.5 + el_pt);
%
% check that point is inside aperture contour
%
				[yy, in] = in_out(xx, d/2, a/2, shape);
				if (in == 1 & abs(y_delta) <= yy),
					n = n + 1;
					el_pts(1,n) = x_center + x_delta;
					el_pts(2,n) = y_delta;
					if (rad ~= 0 & finite(rad)),
					   el_pts(3,n) = abs(el)*om + abs(el)* ...
						(abs(el)-1)*(oo/2 - om) + rad - sign(rad)* ... 
						sqrt(rad^2 - y_delta^2);
					else
					   el_pts(3,n) = abs(el)*om + abs(el)* ...
						(abs(el)-1)*(oo/2 - om) ;
 					end	%ifrad~=0
					el_pts(4,n) = az + 1;
					el_pts(5,n) = el + 3;
				end	%ifin==1
			end	%forel_pt
		    end	%forel
		end	%foraz_pt
	end	%foraz
     else  %if (ROC == 0

% curved array
% ROC > 0 => fixed focus in azimuth
% ROC < 0 => curved array

	n = 0;
	m = 1;
	
	theta_open = 2*asin(d/(2*ROC));
	disp(['Azimuth angle: ',num2str((180/pi)*theta_open),' degrees']);
	d_theta = abs(theta_open)/Nn;

	for az = 0:Nn-1,

		theta_center = d_theta*(-Nn/2 + 0.5 + az);

		temp = [abs(ROC)*sin(theta_center), 0, ROC*(1-cos(theta_center)), az+1];
		centers(1:5,m:m+4) = [temp,1; temp,2; temp,3; temp,4; temp,5]';
		m=m+5;

		for az_pt = 0:nn-1,
			theta_delta =  (d_theta/nn)*(-nn/2 + 0.5 + az_pt);

		    for el = -2:2
			if (el == 0),			% inner elevation element
				mel = mmi;
				d_el = ai;
				rad = ri;
			elseif(abs(el) == 1),		% middle elevation element
				mel = mmm;
				d_el = am;
				rad = rm;
			else				% outer elevation element
				mel = mmo;
				d_el = ao;
				rad = ro;
			end	%ifel=0
			y_center = el*(ai+am)/2 + el*(abs(el)-1)*(ao-ai)/4; %center of
									    %elevation element
			for el_pt = 0:mel-1,
				y_delta = y_center + (d_el/mel)*(-mel/2 + 0.5 + el_pt);
				thet = theta_center + theta_delta;
%
% elevation focusing correction:
%				  
				if (rad ~= 0 & abs(rad) ~= Inf),
				    temp = rad^2 - y_delta^2;
				    if (temp > 0)
					delta_r = abs(el)*om + abs(el)* ...
						(abs(el)-1)*(oo/2 - om) + rad - sign(rad)* ... 
						sqrt(rad^2 - y_delta^2);

				    else
				  	delta_r = abs(el)*om + abs(el)* ...
						(abs(el)-1)*(oo/2 - om) + rad;
				    end  %if (temp > 0)
				else  %if (rad ~= 0
					delta_r = abs(el)*om + abs(el)* ...
						(abs(el)-1)*(oo/2 - om);
				end  %if (rad~= 0
 				xx = (abs(ROC) + delta_r)*sin(thet);
			 				  
%
% check that point is inside aperture contour
%
				[yy, in] = in_out(xx, d/2, a/2, shape);
				if (in == 1 & abs(y_delta) <= yy),
					n = n + 1;
					el_pts(1,n) = xx;
					el_pts(2,n) = y_delta;
	 				el_pts(3,n) = sign(ROC)*(abs(ROC) - ...
				       sqrt(ROC^2 - xx^2)) + delta_r*cos(thet);
					el_pts(4,n) = az + 1;
					el_pts(5,n) = el + 3;
				end  %if (in == 1
			end  %for el_pt
		    end  %for el
		end  %for az_pt
	end  %for az 

    end  %%if (ROC == 0
   	coglob = 'y';
	end


end %if (shape >= 0
		
% ------------------------------------------------------------------

%
% extract elevation points for center azimuth element
% in order to find best fit for ROC (1.5 D)
%
	Nn2 = max(1,fix(Nn/2));
	nnn = find(el_pts(4,:) == Nn2);
	nnnn = nnn(1:length(nnn)/nn);
	yyy = el_pts(2,nnnn);
	zzz = el_pts(3,nnnn);
%
% use fmins to find best fit and initialize with inner lens parameters
%
% function to minimize: circ_fit(x, yyy, zzz) 
%
	
	x0 = [ri , 0];
 	x = x0;
	options = zeros(1,14);
	options(2) = 1.e-8; % termination tolerance for x (parameter vector)
	options(3) = 1.e-8; % termination tolerance for sqsum
	x = fmins('circfit', x0, options, [ ], yyy, zzz);	
%
% give rms
%
	sqsum = sqrt(circfit(x,yyy,zzz)/length(zzz));
	disp(' ');	
  	disp(['Elevation lens curvature: ', num2str(1000*x(1)), ...
              ' mm, z-offset: ',num2str(1000*x(2)),' mm, rms = ', ...
              num2str(sqsum*1000),' mm']) 
%
% display curves
%
		r = x(1);
		z0 = x(2);
		circ = r + z0 -sqrt(r^2 - yyy.^2);
		plot(1000*yyy, 1000*zzz,'*', 1000*yyy, 1000*circ) 
		xlabel(['y [mm], curvature ', num2str(1000*x(1)), ...
                  ' mm, z-offset ',num2str(1000*x(2)),' mm, rms ', ...
                   num2str(sqsum*1000),' mm'])
		if (Mm == 3)
			ylabel(['z [mm], focus ', num2str(1000*ri),', ', ...
                                num2str(1000*ro),' mm, offset ', ...
                                num2str(1000*oo),' mm'])
		else
			ylabel('z [mm]')
		end
		title('Elevation lens and best fitting spherical lens')


	return
 




