%------------------------------------------------------------------------
% Routine         : focannu
%
% Description     : Calculate focusing delays for annular array (elem_pts)
%	 relative to origin, when the sound velocity is c, and focus is given 
%	 as a z-coordinate: z ie. length along z-axis (focus is always on
%	 the z-axis for an annular geometry). 
%	 Unfocused delays, i.e. focus at r = infinity, is obtained by setting
%	 ElF_z = Inf. The components of each elem_pts - point are (x,y,z).
%
% Language        : MATLAB 4.0
% Written by      : Vebjoern Berre 21/4-93
% Version no.     : 1.0 
%                   1.1  SH  02.12.93 Added warning message
%                   1.2  SH  11.11.96 Included Bessel transducers
%                   1.3  SH  21.01.97 excitation is input parameter
%                   1.4  SH  25.03.97 conical focusing
%                   1.5  NAA/SH 26.03.97 Hybrid focusing
%
% Called by       : bpusim
% Revised         : 
%------------------------------------------------------------------------

function [delays,centers] = focannu(excitation,c,elem_pts,transducer)

Npts   = length(elem_pts(1,:));
Nelem  = transducer(2);
d = transducer(1);
F=transducer(9);
ElF_z = excitation(2);

if (length(excitation) < 25) excitation(24:25)=[0 0]; end

%                   

%
% find phase center of each element by averaging over all points in element:
%

n=[1:Nelem];
if (transducer(18) == 0 | transducer(18)==5 | transducer(18)==10),
% 
% equal area
%
	centers_xy=d/2*sqrt((n-.5)./Nelem);
elseif (transducer(18) == 1),
%
% equal width 
%
	centers_xy=(d/2)*(n-.5)./Nelem;

elseif (transducer(18)==3 | transducer(18) == 6)
%
% arbitrary width
% The width of each ring is not stored in a variable
% The following should however find the centers to a good
% approximation if sufficent no. of points are used

	for nn = 1:Nelem
	   ind = find(elem_pts(4,:)==nn);
	   minwidth = ...
	      min(sqrt(elem_pts(1,ind).^2 + elem_pts(2,ind).^2));
	   maxwidth = ...
	      max(max(abs(elem_pts(1,ind))),max(abs(elem_pts(2,ind))));
	   centers_xy(nn) = (minwidth + maxwidth)/2;
	end
else
	centers_xy = zeros(1,Npts);
	disp('Unknown circular transducer. Zero delays used!')
end

%
% calculate delays, dt
%
if transducer(18) ~= 10
%
% spherically shaped transducer
%
  if finite(F)
    centers_z=F*(1-sqrt(1-(centers_xy./F).^2));
  elseif F==inf
    centers_z=zeros(size(centers_xy));
  end
else
%
% conically shaped transducer
%

  centers_z = sin(transducer(12)) * centers_xy;
end


if (excitation(24) == 2)
%
% hybrid focusing (axicon and spherical)
%

   ax_ang = excitation(25);
   for nn= 1:Nelem
     x = centers_xy(nn); z = centers_z(nn);
     if (x >= 0) 			% positive x
       dt(nn) = (z-x*tan(ax_ang))/(2*c);
     else
       dt(nn) = (z-x*tan(-ax_ang))/(2*c);
     end
   end

   
   if finite(ElF_z)
     dist2foc=sqrt((ElF_z-centers_z).^2+centers_xy.^2);
     dr=dist2foc-ElF_z;
     dt=dt-dr/(2*c);
   elseif ElF_z==Inf
     dr=-centers_z;
     dt=dt-dr/(2*c);
   end


elseif excitation(24) == 1,
%
% conical focusing (axicon)
%
   ax_ang = excitation(25);
   for nn= 1:Nelem
     x = centers_xy(nn); z = centers_z(nn);
     if (x >= 0) 			% positive x
       dt(nn) = (z-x*tan(ax_ang))/c;
     else
       dt(nn) = (z-x*tan(-ax_ang))/c;
     end
   end

   
else % spherical focusing (default)

  if finite(ElF_z)
    dist2foc=sqrt((ElF_z-centers_z).^2+centers_xy.^2);
    dr=dist2foc-ElF_z;
    dt=-dr/c;
  elseif ElF_z==Inf
    dr=-centers_z;
    dt=-dr/c;
  end
end  
%
% assign right values to delays according to values of elem_pts(4,:)%

for n=1:Nelem
  ind=find( elem_pts(4,:)==n );
  if isempty(ind)
    disp(' ')
    disp('---------      WARNING:   ----------')
    disp(['No points in element # ',num2str(n)])
    disp('Increase the number of points!')
    disp(' ')
  else
    delays(ind)=dt(n).*ones(size(ind));
  end
end

centers=[centers_xy;centers_z];