function [time,distance,attenuation] = ...
    t_inhom(transducer,excitation,media,x,y,z,elem_pts,flagg)

%------------------------------------------------------------------------
% Routine     : t_inhom.m
%
% Description : Calculate propagation times between the observation 
%               points (x,y,z) and the transducer points elem_pts
%		in an inhomogenuous (layered) 3D medium.
%                
%               time(n,j)= (time from obs. point n to origo) 
%               - (time from obs.point n to trans. point j) 
%               
%               distance(n,j)=distance from obs.point n to trans. point j
%  
%               Possible attenuation factores:
%               Absorption, reflection loss, area attenuation, cosfactor
%               attenuation(n,j)=attenuation of ray between 
%               obs. point n and ray j 
%
% Language    : Matlab 4.1
% Written by  : Frode Teigen, IBT 
% Version no. : 3.0 FT 15.10.93 First Version
%               3.1 FT 28.09.94 Included attenuation (refl and abs)
%               3.2 LO  8.10.95 Attenuation factors are now as described 
%                               in LO's thesis. attenuation/distance is now
%                               A*cosfactor*dJ in Odegaards thesis.
%               3.3 LO  9.10.95 transducer included in call.  
%                               If testflagg=='on' the attenuation factors 
%                               are plotted.
%               3.4 LO 12.10.95 ok for homog. medium too 
%               3.5 LO 13.12.95 Normalization of the attenuation moved to 
%                               beampat
%               3.6 LO 26.04.96 Correction of okrays such that 
%                               length(attenuation)=length(distance) 
%
%        
% Called by   : beampat
% Calling     : findrays, transk, rayareal, normal
% ----------------------------------------------------------------------
testflagg='av';

% Definition of Parameters
%-------------------------
fid = 1;
f = excitation(1);
problem = media(:,1:8);
alpha = media(:,10);
beta = media(:,11);
Npts = size(elem_pts,2);
time = zeros(length(x),size(elem_pts,2));
distance = zeros(length(x),size(elem_pts,2));

%rref=0.1;
%Dref=0.017;
if flagg(6)~=0 % some attenuation factors included
  attenuation = zeros(length(x),size(elem_pts,2));
else % attenuation neglected
  attenuation = 1;
end


%
% Set attenuation flag
% Defined in musim (and musimtxt)
%
disp('  ')
if fix(flagg(6)/2)~=flagg(6)/2 
  absflagg='on';
  disp('Absorption is included')
else
  absflagg='av';
  disp('Absorption is NOT included')
end
if fix(flagg(6)/4)~=flagg(6)/4 & fix((flagg(6)-1)/4)~=(flagg(6)-1)/4
  reflflagg='on';
  disp('Reflection losses are included')
else
  reflflagg='av';
  disp('Reflection losses are NOT included')
end
if fix((flagg(6))/4)~=0
  cosflagg='on';
  disp('Cosine factor on transducer included')
else
  cosflagg='av';
  disp('Cosine factor on transducer NOT included')
end
if (absflagg=='on')&(reflflagg=='on')&(cosflagg=='on')
  arealattflagg='on';
  disp('Attenuation calculated as in Odegaard`s thesis')
else
  arealattflagg='av';
  disp('1/r attenuation included')
end
disp('  ')


fprintf(fid,['Calculating Propagation Time and Distance... \n ']);
pros = 0;
fprintf(fid,'%6.2f percent \n',pros);

%-------------------------------------------
%|                   MAIN LOOP             |
%| Finding propagation times and distances |
%| from all transducer points to one       |
%| observation point at a time             |
%-------------------------------------------


for n = 1:length(x)
   [xcrosstrt,ycrosstrt,zcrosstrt,okrays] = ...
       findrays([elem_pts(1,:) 0],[elem_pts(2,:) 0],...
       [elem_pts(3,:) 0],x(n),y(n),z(n),problem);
   if (absflagg=='on')|(reflflagg=='on')|(cosflagg=='on')|(arealattflagg=='on')
     if max(okrays)==Npts+1
       okrays2=okrays([1:length(okrays)-1]);
     else
       okrays2=okrays;
     end
     attenuation(n,okrays2) = ones(1,length(okrays2));
   else
     attenuation=1;
   end

 
   %
   % attenuation due to broadening of rays
   %
   if arealattflagg=='on' 
     [arealref,arealend]=rayareal(...
       xcrosstrt(:,okrays2),ycrosstrt(:,okrays2),zcrosstrt(:,okrays2),problem,2);
     attenuation(n,okrays2) = attenuation(n,okrays2).*sqrt(arealref./arealend);
     if testflagg=='on'
       plot(sqrt(xcrosstrt(1,okrays2).^2+ycrosstrt(1,okrays2).^2),...
       sqrt(arealref./arealend),'.')
       title('arealfactor')
       xlabel('off-axis')
       pause
     end
   end
 
 
   %
   % attenuation due to reflection
   %
   if reflflagg=='on';   
     trans_p(okrays2) = transk(xcrosstrt(:,okrays2),...
       ycrosstrt(:,okrays2),zcrosstrt(:,okrays2),media,flagg);
     attenuation(n,okrays2) = attenuation(n,okrays2).*trans_p(okrays2);
     if testflagg=='on'
       plot(sqrt(xcrosstrt(1,okrays2).^2+ycrosstrt(1,okrays2).^2),...
       trans_p(okrays2),'.')
       title('reflfactor')
       xlabel('off-axis')
       pause
     end
   end


   %
   % Calculation of cosfactor
   %
   if cosflagg=='on'
     tmpflagg=0;
     if flagg(2)==3  %annular transducer
       F=transducer(9);
       htype=[2,F,F,F,1,0];
       tmpflagg=1;
     end     
     if flagg(2)==1 % 1D og 2D rektangular transducer
       Fxzplane=transducer(17) % azimuth focus, plan hvis=0
       Fyzplane=transducer(9) % elevation fixed focus
       if Fxzplane==0 
         htype=[2,1000000,Fyzplane,Fyzplane,1,0];
         tmpflagg=1;
       end
     end
     if tmpflagg==1
       [nx,ny,nz] = normal(xcrosstrt(1,:),ycrosstrt(1,:),zcrosstrt(1,:),...
       htype,0);
       xout = xcrosstrt(1,:) - xcrosstrt(2,:);  
       yout = ycrosstrt(1,:) - ycrosstrt(2,:);
       zout = zcrosstrt(1,:) - zcrosstrt(2,:);
       r = sqrt(xout.^2 + yout.^2 + zout.^2);
       xout = xout./r;
       yout = yout./r;
       zout = zout./r;
       cosfactor=(1+xout.*nx + yout.*ny + zout.*nz)/2;
       attenuation(n,okrays2) = attenuation(n,okrays2) .*cosfactor(okrays2);
     else
       disp('Warning from t_inhom: cosfactor not included')
     end
     if testflagg=='on'
       plot(sqrt(xcrosstrt(1,okrays2).^2+ycrosstrt(1,okrays2).^2),...
       cosfactor(okrays2),'.')
       title('cosfactor')
       xlabel('off-axis')
       pause
     end
   end


   %
   % Calculation of time and distance
   %
   xcrossend = xcrosstrt(2:size(xcrosstrt,1),1:Npts);
   ycrossend = ycrosstrt(2:size(ycrosstrt,1),1:Npts);
   zcrossend = zcrosstrt(2:size(zcrosstrt,1),1:Npts);
   if find(okrays==(Npts + 1))~=[]
   	xcrossend_0 = xcrosstrt(2:size(xcrosstrt,1),Npts + 1);
  	ycrossend_0 = ycrosstrt(2:size(ycrosstrt,1),Npts + 1);
   	zcrossend_0 = zcrosstrt(2:size(zcrosstrt,1),Npts + 1);
   	xcrosstrt_0 = xcrosstrt(1:size(xcrosstrt,1) - 1,Npts + 1);
   	ycrosstrt_0 = ycrosstrt(1:size(ycrosstrt,1) - 1,Npts + 1);
   	zcrosstrt_0 = zcrosstrt(1:size(zcrosstrt,1) - 1,Npts + 1);
   	time_0 = sum(sqrt((xcrosstrt_0 - xcrossend_0).^2 +...
			  (ycrosstrt_0 - ycrossend_0).^2 +...
	         	  (zcrosstrt_0 - zcrossend_0).^2)./...
			  (problem(:,2)*ones(1,size(xcrosstrt_0,2))));
    else
	error(' PROGRAM ABORTED - Impossible to find reference time');
    end;
   xcrosstrt = xcrosstrt(1:size(xcrosstrt,1) - 1,1:Npts);
   ycrosstrt = ycrosstrt(1:size(ycrosstrt,1) - 1,1:Npts);
   zcrosstrt = zcrosstrt(1:size(zcrosstrt,1) - 1,1:Npts);
   distances = sqrt((xcrosstrt(:,okrays2) -...
       xcrossend(:,okrays2)).^2 + (ycrosstrt(:,okrays2) -...
       ycrossend(:,okrays2)).^2 + (zcrosstrt(:,okrays2) -...
       zcrossend(:,okrays2)).^2);
   if size(distances,1)==1
     distance(n,okrays2)=distances;
     time(n,okrays2) = distances./... 
       (problem(:,2)*ones(1,size(xcrosstrt(:,okrays2),2)));
   else
     distance(n,okrays2) = sum(distances);
     time(n,okrays2) = sum(distances./... 
     (problem(:,2)*ones(1,size(xcrosstrt(:,okrays2),2))));
   end
   time(n,:) = time_0 - time(n,:);
   if testflagg=='on'
     plot(sqrt(xcrosstrt(1,okrays2).^2+ycrosstrt(1,okrays2).^2),...
     time(n,okrays2),'.')
     title('time  (t0-t)')
     xlabel('off-axis')
     pause
     plot(sqrt(xcrosstrt(1,okrays2).^2+ycrosstrt(1,okrays2).^2),...
     distance(n,okrays2),'.')
     title('distance')
     xlabel('off-axis')
     pause
   end
   

   %
   % attenuation due to absorption
   %
   if absflagg=='on'
     alph = alpha*ones(1,size(xcrosstrt(:,okrays2),2));
     bet = beta*ones(1,size(xcrosstrt(:,okrays2),2));
     intatt = exp(-alph.*((f/1e6).^bet).*distances);
     if size(distances,1)==1
       abstmp=sqrt(intatt);
     else
       abstmp=prod(sqrt(intatt));
     end
     attenuation(n,okrays2) = attenuation(n,okrays2) .*abstmp;
     if testflagg=='on'
       plot(sqrt(xcrosstrt(1,okrays2).^2+ycrosstrt(1,okrays2).^2),abstmp,'.')
       title('absfactor')
       xlabel('off-axis')
       pause
     end
   end


   %
   % weighting each ray with its corresponding transducer areal
   %
   %Dref=0.017; flyttet opp
   %arealref=pi*(Dref/2)^2;
   if flagg(2)==3  %annular transducer
     Dtrans=transducer(1);
     %ntranspoint=transducer(3);
     ntranspoint=size(elem_pts,2);
     arealperray=pi*(Dtrans/2)^2/ntranspoint;
   elseif flagg(2)==1 % 1D og 2D rektangular
     %ntranspoint=transducer(2)*transducer(3)*transducer(6)*transducer(7);
     ntranspoint=size(elem_pts,2);
     arealperray=transducer(1)*transducer(5)/ntranspoint;
   else
     disp('Warning in t_inhom: areal to transducer unknown')
     arealperray=1;
   end
   %arealperray=arealperray/arealref;
   if (absflagg=='on')|(reflflagg=='on')|(cosflagg=='on')|(arealattflagg=='on') 
     attenuation(n,okrays2)=attenuation(n,okrays2)*arealperray;
   else
     attenuation=attenuation*arealperray;
   end


   %
   % correction/ triksing  for aa tilpasse seg beampat
   % attenuation/distance is now = A*cosfactor*dJ in Odegaards thesis
   %
   if arealattflagg=='on' 
     attenuation(n,okrays2) = attenuation(n,okrays2) .*distance(n,okrays2);
   end
   if testflagg=='on'
     plot(sqrt(xcrosstrt(1,okrays2).^2+ycrosstrt(1,okrays2).^2),...
     attenuation(n,okrays2),'.')
     title('tot. attenuation')
     xlabel('off-axis')
     pause
     ntranspoint=size(elem_pts,2);
     plot(sqrt(xcrosstrt(1,okrays2).^2+ycrosstrt(1,okrays2).^2),...
     attenuation(n,okrays2)./distance(n,okrays2)*ntranspoint,'.')
     title('tot. attenuation/distance*nrays')
     xlabel('off-axis')
     pause
   end

 
   %
   % rays with errors
   %
   ind = find(distance(n,:)==0);
   if ind~=[]
     distance(n,ind) = -1e-10*ones(size(ind));
     pros = length(ind)/Npts*100;
     fprintf(fid,...
     '\n Warning! No solution for %4.2f percents of the transducer points',...
     pros);
   end
   pros = n/(length(x))*100;
   fprintf(fid,'\n %6.2f percents \n',pros);
end



 















