%------------------------------------------------------------------------
% Routine     : ultremez.m
% Description : Uniform Remez exchange algorithm of UltraSim
%               This algorithm is a simplified version of Parks & McClellands
%               REMEZ algorithm and is based partially on the version of
%               MathWorks, Inc.
% Language    : Matlab 4.1
% Written by  : Jan Ove Erstad , IfI-UiO
%
% Version no. : 1.0 JOE 09.09.94 First version, plots polynomial in each step
%               1.1 BJE 25.04.95 removed ConfigOrigHandle from functioncall
%
%
% Called by   : usimcnf
% Calling     : remezdd
%------------------------------------------------------------------------
function [out] = ultremez(lgrid,elem_pts,transducer,option,media,...
      excitation,flagg,config)
%
  
% INITIALIZE
  lgrid	=  16; 				% grid density
  lambda =  media(1,2)/excitation(1); 	% wavelength
  xi     =  elem_pts(1,(elem_pts(4,:) >= 0))./lambda; %normalized wavenumber
  
% USER INPUT
  angl = input('Give initial cut angle (stopband): ');
  if isempty(angl)
      disp('Unknown command!');
      disp('Input format is cut-angle i degrees.');     
    else
      if (angl < 3) | (angl > 50)
          disp('Illegal input characteristics!');
          disp('Input should be 3 < cut-angle < 50');
          return
      end
      ff = [radians(1) radians(angl)]; 			% normalized frequency
      K  = 10;
  end
  
  
  if (max(size(elem_pts))	~= max(size(elem_pts(1,elem_pts(4,:) >= 0 ))))
      disp(' This routine demands a filled array!');
      disp(' Return!');
      break
  end
  
  nfilt	= max(size(elem_pts));
  ff  = sin([0 ff pi/2]); 		% cut-off angles/frequencies 
  aa  = [1 1 0 0]; 			%desired amplitude
  wtx = [1 K]; 				%weighting of errors
  
  
  if nfilt < 4 				%check if array is shorter than minimum
      error('Number of elements must be at least 3!');
  end
  
% INTERPOLATE CUT OFF ANGLES INTO LARGE GRID
  edge = ff;
  edge = edge/2;
  cmptr= computer;
  
%check if even or odd numbered array
  nodd	=  rem(nfilt,2);	% nodd == 0 -> filter length is even
% nodd == 1 -> filter length is odd
  nfcns	=  fix(nfilt/2);	% number of basis functions
  if nodd == 1 
      nfcns = nfcns + 1;
  end
  
% SET UP THE DENSE GRID
  dgrid(1) =  edge(1);
  delf  = 0.5/(lgrid*nfcns);
  fup   = edge(2);
  dgrid = [dgrid (dgrid(1)+delf):delf:(fup+delf)];
  jup   = max(size(dgrid));
  dgrid(jup-1) = fup;
  sel   = 1:jup-1;
% The desired magnitude function (des) and weigth function (wt) for the 
% pass band and stop band are the lines connecting the points
  des(sel)= polyval(aa(1),dgrid(sel));
  wt(sel) = wtx(1)./ones(size(sel));
  
  j     = jup;
  dgrid(j) = edge(3);
  fup   = edge(4);
  dgrid = [dgrid (dgrid(j)+delf):delf:(fup+delf)];
  jup = max(size(dgrid));
  dgrid(jup-1) = fup;
  sel        =	j:jup-1;
  des(sel)= polyval(aa(3),dgrid(sel));
  wt(sel) = wtx(2)./ones(size(sel));
  
  ngrid = jup - 1;
  if nodd == 0 & dgrid(ngrid) > .5-delf
      ngrid = ngrid -1;
  end
  
  if nodd ~= 1
% When array is of even length => set up a NEW APPROXIMATION PROBLEM 
% according to Eq.7.127-8 in Oppenheim&Schafer (O&S)
      des = des(1:ngrid)./cos(pi*dgrid(1:ngrid));
      wt  = wt(1:ngrid).*cos(pi*dgrid(1:ngrid));
  end
  
% INITIAL GUESS FOR THE EXTREMAL FREQUENCIES 
%	- equally spaced along the dgrid pattern
  temp = (ngrid - 1)/nfcns;
  j    = 1:nfcns;
  iext = fix([(j-1)*temp+1 ngrid])'; 	%'
  nm1  = nfcns - 1;
  nz   = nfcns + 1;
  
%
% THE REMEZ EXCHANGE ALGORITHM
%
% initialization 
  comp	=  [];
  itrmax	=  25;
  devl	=  -1;
  nzz	=  nz + 1;
  niter	=  0;
  jchnge	=  1;
  jet	=  fix((nfcns - 1)/15) + 1;
  
% outer exchange loop
  while jchnge > 0
      iext(nzz)	=  ngrid + 1;
      niter	=  niter + 1;
      if niter > itrmax
          break
      end
      
      l	=  iext(1:nz)'; 		%'
      x	=  cos(2*pi*dgrid(l));
      ll    =  iext(1:nz);
      
% Calculate the Lagrange interpolation coeffisients to be used 
% in the GEE part by using Eq. 7.132 in O&S
      for nn = 1:nz
          ad(nn)	= remezdd(nn,nz,jet,x);
      end
      
      add	= ones(size(ad));
% making an alternation vector
      add(2:2:nz) = -add(2:2:nz);
      dnum	=  ad*des(l)'; 			%'
      dden	=  add*(ad./wt(l))'; 		%'
% calculate the deviation using Eq.7.131 in O&S
      dev	= dnum/dden; 			% line 131
      nu = 1;
      
      if dev > 0
          nu = -1;
      end
      
      dev	= -nu*dev;
% Eq.7.133b in O&S
      y	= des(l) + nu*dev*add./wt(l);
      
% Check if the deviation is less than the deviation limit (-1)
      if dev <= devl
          disp('The algorithm does not converge!');
          break
      end
      
      
% set new deviation limit
      devl = dev; 				%line 150
      jchnge  = 0;
      k1	=  iext(1);
      knz	=  iext(nz);
      klow=  0;
      nut	= -nu;
      j	=  1;
      flag34  = 1;
      
% SEARCH FOR THE EXTREMAL FREQUENCIES OF THE BEST APPROXIMATION
% 
      
      while j < nzz 			% inner loop 1
          kup  =  iext(j+1);
          l    =	iext(j) + 1;
          nut  =	-nut;
          if j == 2
              y1 = comp;
          end
          comp = dev;
          flag = 1;
          if l < kup
% gee
              c = ad./(cos(2*pi*dgrid(l))-x);  
              err = (c*y'/sum(c) - des(l))*wt(l); %'(P(cosw)-D)*W=error
              dtemp = nut*err - comp;
              if dtemp > 0				
% error -E > calculated deviation -delta
                  comp = nut*err;
                  l = l + 1;
                  while l < kup
% gee
                      c = ad./(cos(2*pi*dgrid(l))-x);  
                      err = (c*y'/sum(c) - des(l))*wt(l); %'
                      dtemp = nut*err - comp;
                      if dtemp > 0
                          comp = nut*err;
                          l = l + 1;
                        else
                          break;
                      end
                  end    
                  iext(j) = l - 1;
                  j = j + 1;
                  klow = l - 1;
                  jchnge = jchnge + 1;
                  flag = 0;
              end
          end
          if flag
              l = l - 2;
              while l > klow
% gee
                  c = ad./(cos(2*pi*dgrid(l))-x);  
                  err = (c*y'/sum(c) - des(l))*wt(l);%'
                  dtemp = nut*err - comp;
                  if dtemp > 0 | jchnge > 0
                      break;
                  end
                  l = l - 1;
              end
              if l <= klow
                  l = iext(j) + 1;
                  if jchnge > 0
                      iext(j) = l - 1;
                      j = j + 1;
                      klow = l - 1;
                      jchnge = jchnge + 1;
                    else
                      l = l + 1;
                      while l < kup
% gee
                          c = ad./(cos(2*pi*dgrid(l))-x);  
                          err = (c*y'/sum(c) - des(l))*wt(l); %'
                          dtemp = nut*err - comp;
                          if dtemp > 0
                              break;
                          end
                          l = l + 1;
                      end
                      if l < kup & dtemp > 0
                          comp = nut*err;
                          l = l + 1;
                          while l < kup
% gee
                              c = ad./(cos(2*pi*dgrid(l))-x);  
                              err = (c*y'/sum(c) - des(l))*wt(l); %'
                              dtemp = nut*err - comp;
                              if dtemp > 0
                                  comp = nut*err;
                                  l = l + 1;
                                else
                                  break;
                              end
                          end    
                          iext(j) = l - 1;
                          j = j + 1;	
                          klow = l - 1;
                          jchnge = jchnge + 1;
                        else
                          klow = iext(j);
                          j = j + 1;
                      end
                  end
                elseif dtemp > 0
                  comp = nut*err;
                  l = l - 1;
                  while l > klow
% gee
                      c = ad./(cos(2*pi*dgrid(l))-x);  
                      err = (c*y'/sum(c) - des(l))*wt(l); %'
                      dtemp = nut*err - comp;
                      if dtemp > 0
                          comp = nut*err;
                          l = l - 1;
                        else
                          break;
                      end
                  end
                  klow = iext(j);
                  iext(j) = l + 1;
                  j = j + 1;
                  jchnge = jchnge + 1;
                else
                  klow = iext(j);
                  j = j + 1;
              end
          end 				%if flag 
      end 					%while j < nzz --> end of inner loop 1
      
      while j == nzz 			% inner loop 2  	
          ynz = comp;
          k1 = min([k1 iext(1)]);
          knz = max([knz iext(nz)]);
          nut1 = nut;
          nut = -nu;
          l = 0;
          kup = k1;
          comp = ynz*1.00001;
          luck = 1;
          flag = 1;
          l = l + 1;
          while l < kup 			%subloop 2.1
% gee
              c = ad./(cos(2*pi*dgrid(l))-x);  
              err = (c*y'/sum(c) - des(l))*wt(l); %'
              dtemp = err*nut - comp;
              if dtemp > 0
                  comp = nut*err;
                  j = nzz;
                  l = l + 1;
                  while l < kup 			%subloop 2.2
% gee
                      c = ad./(cos(2*pi*dgrid(l))-x);  
                      err = (c*y'/sum(c) - des(l))*wt(l); %'
                      dtemp = nut*err - comp;
                      if dtemp > 0
                          comp = nut*err;
                          l = l + 1;
                        else
                          break;
                      end
                  end 				% end subloop 2.2  
                  iext(j) = l - 1;
                  j = j + 1;
                  klow = l - 1;
                  jchnge = jchnge + 1;
                  flag = 0;
                  break;
              end
              l = l + 1;
          end 				% end subloop 2.1
          
% l >= kup
          if flag
              luck = 6;
              l = ngrid + 1;
              klow = knz;
              nut = -nut1;
              comp = y1*1.00001;
              l = l - 1;
              while l > klow 			%subloop 2.3 
% gee
                  c = ad./(cos(2*pi*dgrid(l))-x);  
                  err = (c*y'/sum(c) - des(l))*wt(l); %'
                  dtemp = err*nut - comp;
                  if dtemp > 0
                      j = nzz;
                      comp = nut*err;
                      luck = luck + 10;
                      l = l - 1;
                      while l > klow 		%subloop 2.4
% gee
                          c = ad./(cos(2*pi*dgrid(l))-x);  
                          err = (c*y'/sum(c) - des(l))*wt(l);%'
                          dtemp = nut*err - comp;
                          if dtemp > 0
                              comp = nut*err;
                              l = l - 1;
                            else
                              break;
                          end
                      end 				% end subloop 2.4	
                      klow = iext(j);
                      iext(j) = l + 1;
                      j = j + 1;
                      jchnge = jchnge + 1;
                      flag = 0;
                      break;
                  end
                  l = l - 1;
              end 				% end subloop 2.3
              if flag
                  flag34 = 0;
                  if luck ~= 6
                      iext = [k1 iext(2:nz-nfcns)' iext(nz-nfcns:nz-1)']'; %'
                      jchnge = jchnge + 1;
                  end
                  break;
              end
          end
      end 					% END inner loop 2
      
      if flag34 & j > nzz 
          if luck > 9
              iext = [iext(2:nfcns+1)' iext(nfcns+1:nz-1)' iext(nzz) iext(nzz)]';%'
              jchnge = jchnge + 1;
            else
              y1 = max([y1 comp]);
              k1 = iext(nzz);
              l = ngrid + 1;
              klow = knz;
              nut = -nut1;
              comp = y1*1.00001;
              l = l - 1;
              while l > klow 			% inner loop 3
% gee
                  c = ad./(cos(2*pi*dgrid(l))-x);  
                  err = (c*y'/sum(c) - des(l))*wt(l); %'
                  dtemp = err*nut - comp;
                  if dtemp > 0
                      j = nzz;
                      comp = nut*err;
                      luck = luck + 10;
                      l = l - 1;
                      while l > klow 		% subloop 3.1
% gee
                          c = ad./(cos(2*pi*dgrid(l))-x);  
                          err = (c*y'/sum(c) - des(l))*wt(l); %'
                          dtemp = nut*err - comp;
                          if dtemp > 0
                              comp = nut*err;
                              l = l - 1;
                            else
                              break;
                          end
                      end 				% end subloop 3.1
                      klow = iext(j);
                      iext(j) = l + 1;
                      j = j + 1;
                      jchnge = jchnge + 1;
                      iext = [iext(2:nfcns+1)' iext(nfcns+1:nz-1)' iext(nzz) iext(nzz)]';%'
                      break;
                  end
                  l = l - 1;
              end 				% end innerloop 3
              
              if luck ~= 6
                  iext = [k1 iext(2:nz-nfcns)' iext(nz-nfcns:nz-1)']';%'
                  jchnge = jchnge + 1;
              end
          end 				% if luck >9 
      end 					% if flag34 & j>nzz
  end 					% end outer exchange loop
  
% CALCULATION OF THE COEFFISIENTS OF THE BEST APPROXIMATION 
% USING THE INVERSE DISCRETE FOURIER TRANSFORM
  nm1   = nfcns - 1;
  fsh   = 1.0e-6;
  gtemp = dgrid(1);
  x(nzz)= -2;
  cn    = 2*nfcns - 1;
  delf  = 1/cn;
  l = 1;
  
% Calculate the a- coefficients
  for j = 1:nfcns
      ft = (j-1)*delf;
      xt = cos(2*pi*ft);
      xe = x(l);
      while xt <= xe & xe-xt >= fsh
          l = l + 1;
          xe = x(l);
      end
% if approx. at an extremal frequency
      if abs(xt-xe) < fsh
          a(j) = y(l);
          else	
          dgrid(1) = ft;
% gee
          c = ad./(cos(2*pi*ft)-x(1:nz));  
          a(j) = c*y'/sum(c); 		%'
      end
      
      l = max([1, l-1]);
  end 					%for j=1:nfcns
  dgrid(1) = gtemp;
  dden = 2*pi/cn;
% Calculate the alpha- coefficients from the a- values
  for j = 1:nfcns
      dnum = (j-1)*dden;
      if nm1 < 1
          alpha(j) = a(1);
        else
          alpha(j) = a(1) + 2*cos(dnum*(1:nm1))*a(2:nfcns)'; %'
      end
  end
% Making alpha a column vector and dividing it by the length of the array -1
  alpha = [alpha(1) 2*alpha(2:nfcns)]'/cn; %'
  if nfcns <= 3
      alpha(nfcns + 1) = 0;
      alpha(nfcns + 2) = 0;
  end
  
% Making alpha a row vector again
  alpha=alpha'; 				%'
  
% VISUALIZE THE ALGORITHM IN EACH STEP
  fr=dgrid;
  stepfu =  2*pi*[0:(nfcns-1)];
  
  for lk=1:max(size(fr))
      cosfu  =  [cos(stepfu.*fr(lk))];
      if nodd ~= 0
          FR(lk) = sum(alpha.*cosfu);
        else
          FR(lk) = cos(pi*fr(lk))*sum(alpha.*cosfu);
      end
  end
  
  subplot(1,1,1);
  plot(180/pi*asin(2*fr),20*log10(abs(FR)/max(abs(FR))),'r');
  grid on 
  title('COS-POLYNOM');
  hold on
  
  fre=dgrid(ll);
  stepfu =  2*pi*[0:(nfcns-1)];
  for km=1:max(size(ll))
      if nodd ~= 0  
          frr(km)= sum(alpha.*cos(stepfu.*fre(km)));
        else
          frr(km)= cos(pi*fre(km))*sum(alpha.*cos(stepfu.*fre(km)));
      end
  end
  plot(180/pi*asin(2*fre),20*log10(abs(frr)/max(abs(frr))),'cx');
  
  ijext=iext(1:nz);
  fl=dgrid(ijext);
  for kl=1:max(size(fl))
      if nodd ~= 0 
          flr(kl)= sum(alpha.*cos(stepfu.*fl(kl)));
        else
          flr(kl)= cos(pi*fl(kl))*sum(alpha.*cos(stepfu.*fl(kl)));
      end
  end
  plot(180/pi*asin(2*fl),20*log10(abs(flr)/max(abs(flr))),'yo');
  hold off
%end 					% end outer exchange loop
  
% CALCULATE THE IMPULSE RESPONSE
  if nodd ~= 0
      h = [.5*alpha(nz-1:-1:nz-nm1) alpha(1)];
    else
      
      h = .25*[alpha(nfcns)  ...
          alpha(nz-2:-1:nz-nm1)+alpha(nfcns:-1:nfcns-nm1+2)...
          2*alpha(1)+alpha(2)];
  end
  hold off
  
  lenh=max(size(h));
  h = [h(1:lenh-nodd) h(lenh:-1:1)];
  out = h/max(abs(h));
  
  subplot(2,1,1); 			% plot apodization
  plot(xi,out); grid
  ylabel('Weighting');
  xlabel('Element Positions in Wavelengths');
  
  subplot(2,1,2); 			% plot polynomial values
  lambda =  media(1,2)/excitation(1); 	% wavelength
  xi     =  2*elem_pts(1,(elem_pts(1,:) >= 0))./lambda; %normalized wavenumber
  uaxis = pi*(0:.002:1);
  for lk=1:max(size(uaxis))
      cosfu  =  [cos(xi.*uaxis(lk))]; 	% the basis functions
      Resp(lk) = sum(alpha.*cosfu);
  end
%keyboard
  plot(uaxis/pi,20*log10(abs(Resp)/abs(Resp(1))),'r'); grid on 
  xlabel('sin(phi)');
  ylabel('Response [dB]');
  axis([0 1 -80 0]);
  fprintf('\nFinished!\n');
  disp('Push a key to continue. ');
  ginput(1);
%------------------------------------------------------------------------
  
