%------------------------------------------------------------------------------
% Routine          : rectangu()
%
% Description      : place elements and points on surface of array.
%		     surface is rectangular or oval
%		     Array may be planar or curved and/or have fixed focus
%                    in azimuth/elevation.
%		     The points are placed according to a uniform grid
%		     1.5D Array : The number of points on each element in
%		     elevation is proportional to the aperture of the element.
%		     The points on each element are placed according to 
%		     a uniform grid
%		     
% Limitations: 	     
%
% Idea:	  1 elevation element => 1D array
%		     3,5 el elements     => 1.5 D array	
%		     all other cases     => 2D array
%                    
% Output data structure:
%			Point information:
% 		     el_pts(1,n) = x
% 		     el_pts(2,n) = y
% 		     el_pts(3,n) = z 
% 		     el_pts(4,n) = nn % element number in major dimension (azimuth)
% 		     el_pts(5,n) = mm % element number in minor dimension (elevation)
%                    centers(1:5,n) = similar to el_pts except is per element
%
%
% Language         : Matlab syntax
% Written by       : Kari Lervik, September 1992 
%
% Version no.      : 1.0
% Version no.      : 1.1 SH  20.03.93 completely rewritten 
% Version no.      : 1.2 SH  12.05.93 updated
% Version no.      : 1.3 TKL 14.07.93 put fid=1 in file for unifying printouts
% Version no.	    : 1.4 FT  27.07.93 1.5D Array
% Version no.	    : 1.5 TKL 21.10.93 Corrected elevation point pitch 
%                  : 1.6 SH   3.12.93 Also corrected elevation point pitch!
%                                     Added routines for finding fit to el. lens
%                  : 1.7 SH   4. 2.94 Printout of elevation Fresnel number 
%		             : 1.8 JOE 18.02.94 2D array (not curved)
%		             : 1.9 JOE 14.03.94 Mills cross 1 & 2
%		             : 2.0 SH  09.08.94 corrected small bug in elevation lens
%                                     curvature output
%                  : 2.1 SH   6.12.94 Shortened from rectangular to rectangu
%                  : 2.2 KE   2. 5.96 Octogonal transducer p = -8
%                  : 2.3 SH   8. 5.96 Added check of negative shape parameter
%                                     and corrected memory allocation for elem_pts
%                  : 2.4 SH  03.06.96 Moved octagonal to in_out.m
%                                     Moved 1.5d -> set15d.m
%                                     Moved Mills cross -> mills.m  
%                                     Computed and output centers of
%                                     elements  
%                  : 2.5 SH  14.11.96 Corrected size of centers for
%                                     non-rectangular shapes and 2D arrays                 
%						 : 2.6 JEK 10.07.00 Curved array
%
% Called by        : t1usim.m, usiminit.m, lusim.m
% Routines called  : rcurved.m, in_out.m, set15d.m, mills.m 
% Load files       :
%
%------------------------------------------------------------------------------

function [el_pts,centers] = rectangu(transducer,excitation,media);

% TKL 14/7-93
fid=1;
% VEB 17/3-93
a=transducer(5); % Elevation array-aperture
ai=transducer(12); % Elevation array-apertur, inner
am=transducer(13); % Elevation array-apertur, middle
ao=transducer(14); % Elevation array-apertur, outer
Mm=transducer(6); % Elevation # elements
Nn=transducer(2); % Azimuth # elements
om=transducer(15);
oo=transducer(16);
ri=transducer(9);
rm=transducer(10);
ro=transducer(11);
d=transducer(1); % Azimuth array-apperture
% SH 19.3.93:
mm = transducer(7); % Elevation # points
nn = transducer(3); % Azimuth # points per element
ROC = transducer(17);
shape = transducer(18);

npts = Nn*nn*mm; % total number of points in 1D rectangular array
disp(' ')
disp(' Finding element and point coordinates .....................')
disp(' ')

% JOE 14/3-94
if (shape >= 0 | shape == -8)
   if Mm == 1
      
      % SH 19. March 1993:
      % Single elevation element modified for rectangular grid
      % elevation focusing and curved array
      % and new data structure
      
      % find element spacing in azimuth and elevation:
      d_az = d/Nn;
      d_el = ai;
      
      % allocating memory
      npts = Nn*nn*mm; % total number of points i 1D rectangular array
      if (shape == 0)
         el_pts = zeros(5,npts-10);
         centers = zeros(5,Nn*Mm);
      else
         el_pts = zeros(5,round(2*npts/3));
         centers = zeros(5,2*Nn*Mm/3);
      end
      
      % rectangular array:
      if (ROC == 0 | abs(ROC) == Inf)
         n = 0;
         for az = 0:Nn-1
            x_center = d_az*(-Nn/2 + 0.5 + az);
            centers(1:5,az+1) = [x_center, 0, 0, az+1, 1]';
            for az_pt = 0:nn-1
               x_delta =  (d_az/nn)*(-nn/2 + 0.5 + az_pt);
               xx = x_center + x_delta;
               
               % check that point is inside aperture contour
               [yy, in] = in_out(xx, d/2, a/2, shape);
               for el_pt = 0:mm-1
                  y_delta = (d_el/mm)*(-mm/2 + 0.5 + el_pt);
                  [yy, in] = in_out(xx, d/2, a/2, shape);
                  if (in == 1 & abs(y_delta) <= yy),
                     n = n + 1;
                     el_pts(1,n) = x_center + x_delta;
                     el_pts(2,n) = y_delta;
                     if (ri ~= 0 & finite(ri)),
                        el_pts(3,n) = ri - sign(ri)*sqrt(ri^2 - y_delta^2);
                     else
                        el_pts(3,n) = 0;
                     end % if (ri ~= 0)
                     el_pts(4,n) = az + 1;
                     el_pts(5,n) = 1;
                  end  %if (in == 1)
               end  %for el_pt = 0
               
            end % for az_pt = 0
         end % for az = 0
         
      else  %if (ROC == 0)
         
         % curved array
         % ROC > 0 => fixed focus in azimuth
         % ROC < 0 => curved array
         
         n = 0;
         
         theta_open = 2*asin(d/(2*ROC));
         disp(['Azimuth angle: ',num2str((180/pi)*theta_open),' degrees']);
         d_theta = abs(theta_open)/Nn;
         
         for az = 0:Nn-1,
            theta_center = d_theta*(-Nn/2 + 0.5 + az);
            centers(1:5,az+1) = [abs(ROC)*sin(theta_center), 0,...
                  ROC*(1-cos(theta_center)), az+1, 1]';
            for az_pt = 0:nn-1
               theta_delta =  (d_theta/nn)*(-nn/2 + 0.5 + az_pt);
               for el_pt = 0:mm-1
                  y_delta = (d_el/mm)*(-mm/2 + 0.5 + el_pt);
                  thet = theta_center + theta_delta;
                  
                  % elevation focusing correction:
                  if (ri ~= 0 & abs(ri) ~= Inf),
                     temp = ri^2 - y_delta^2;
                     if (temp > 0)
                        delta_r = ri - sign(ri)*sqrt(temp);
                     else
                        delta_r = ri;
                     end % if (temp > 0)
                  else % if (ri ~= 0)
                     delta_r = 0;
                  end % if (ri ~= 0)
                  xx = (abs(ROC) + delta_r)*sin(thet)
                  
                  % check that point is inside aperture contour      
                  [yy, in] = in_out(xx, d/2, a/2, shape);
                  if (in == 1 & abs(y_delta) <= yy),
                     n = n + 1;
                     el_pts(1,n) = xx;
                     el_pts(2,n) = y_delta;
                     el_pts(3,n) = sign(ROC)*(abs(ROC) - sqrt(ROC^2 - xx^2)) + ...
                        delta_r*cos(thet);
                     el_pts(4,n) = az + 1;
                     el_pts(5,n) = 1;
                  end % if (in == 1)
               end % for el_pt = 0
            end % for az_pt = 0
         end % for az = 0
         
      end   %if (ROC == 0)
      
   elseif (Mm == 3 | Mm == 5),
      
      % 1.5 D array
      [el_pts, centers] = set15d(transducer,excitation,media); 
      
   else
      % --------------------------------------------------------------------
      % 	JOE 18.02.1994
      % 	2D-array   i.e. 2,4 or >5 elements in elevation dimension
      % --------------------------------------------------------------------
      % uniform element spacing in azimuth dimension
      d_az = d/Nn;
      
      % uniform element spacing in elevation dimension
      d_el = a/Mm;
      
      % number of points in elevation dimension
      mmm = round(mm/Mm);
      
      if mmm > 0 
         disp(['Number of points per elevation element: ',num2str(mmm),'']);
      else
         disp('Warning!! Not enough points to fill all elements!');
         disp('Calculation is terminated!');
         return
      end 
      
      npts = Nn*nn*mmm*Mm; 
      
      % allocating memory
      if (shape == 0)
         el_pts = zeros(5,npts-10);
         centers = zeros(5,Nn*Mm);	
      else
         el_pts = zeros(5,round(2*npts/3));
         centers = zeros(5,2*Nn*Mm/3);	
      end
      
      if (ROC == 0) | (abs(ROC) == inf),
         
         % rectangular array
         n=0;
         m=0;
         for az = 0:Nn-1,
            x_center = d_az*(-Nn/2 + .5 + az);
            
            for az_pt = 0:nn-1,
               x_delta = (d_az/nn)*(-nn/2 + .5 + az_pt);
               xx = x_center + x_delta;
               
               % check if point is inside aperture
               [yy,in] = in_out(xx,d/2,a/2,shape);
               for el = 0:Mm-1
                  
                  % center of elevation element
                  y_center = d_el*(-Mm/2 + .5 + el);
                  
                  % added check for inside/outside 14.11.96 SH
                  if (az_pt == 0 & (in == 1) & (abs(y_center) <= yy)),
                     m = m+1;	
                     if (ri ~= 0) & finite(ri),
                        zzz = ri - sign(ri)*sqrt(ri^2 - y_center^2);
                     else
                        zzz = 0;
                     end
                     centers(1:5,m) = [x_center, y_center, zzz, az+1,el+1]';
                  end
                  
                  for el_pt = 0:mmm-1
                     y_delta = y_center+(d_el/mmm)*(-mmm/2 + .5 + el_pt);
                     [yy,in] = in_out(xx,d/2,a/2,shape);
                     
                     if (in == 1) & (abs(y_delta) <= yy)
                        n = n + 1;
                        el_pts(1,n) = x_center + x_delta;
                        el_pts(2,n) = y_delta;
                        
                        % z-coordinates if array is prefocused
                        if (ri ~= 0) & finite(ri),
                           el_pts(3,n) = ri - sign(ri)*sqrt(ri^2 - y_delta^2);
                        else
                           % z-coordinates planar array
                           el_pts(3,n) = 0;
                        end  %if ((ri ~=
                        el_pts(4,n) = az + 1;
                        el_pts(5,n) = el + 1;
                     end % if (in == 1) ...
                  end % for el_pt = 0:mmm
               end % for el = 0:Mm
            end % for az_pt = 0:n
         end % for az = 0
         
      else
         % ROC > 0 => fixed focus in azimuth
         % ROC < 0 => curved array
         mmm = round(mm/Mm);
         if mmm > 0 & ~mod(mm,Mm)
            disp(['Number of points per elevation element: ',num2str(mmm),'']);
            [el_pts,centers,theta_open] = rcurved(transducer);
         else
            disp('Warning!! Ratio points/elements must be an integer');
            disp('greater than zero! Calculation is terminated!');
            return
         end
      end % if ROC == 0
      
   end % else 2D-array
   
elseif (shape == -1 | shape == -2) % Mills' Cross
   el_pts = mills(transducer,excitation,media); 
   centers = el_pts;
else
   disp(['Illegal transducer shape parameter (transducer(18) = ',num2str(shape),')'])
   return   
end % if (shape >= 0)

% ------------------------------------------------------------------

% el_pts
% check memory allocation and remove inactive points
inactive = find(el_pts(4,:) == 0);
el_pts(:,inactive) = [];

disp(' ')
disp(['Number of points: ',num2str(max(size(el_pts(1,:))))])

% compute pitch (element distance)
lambda = media(1,2)/excitation(1);

% azimuth
if (ROC == 0 | ROC == inf),
   pitch = d/Nn ;
else
   pitch = abs(ROC*theta_open/Nn);
end
point_pitch = pitch/nn;

% elevation
p_el = a/Mm;
point_el = a/mm;

fprintf(fid,['\n']);
fprintf(fid,['\t    Element pitch [mm]\t Element pitch [lambda]\t Point pitch [lambda]\n']);

fprintf(fid,['Azimuth:\t',num2str(1000*pitch),'\t\t\t', ... 
      num2str(pitch/lambda),'\t\t\t', ...
      num2str(point_pitch/lambda),'\n']); 
fprintf(fid,['Elevation:\t',num2str(1000*p_el),'\t\t\t', ... 
      num2str(p_el/lambda),'\t\t\t', ...
      num2str(point_el/lambda),'\n']); 

if (Mm == 1 & shape ~= -1 & shape ~=-2),
   
   % 		elevation Fresnel number
   S_elev = 4*ri * lambda/(ai*ai);
   Int_max = ri/(1+S_elev/2);
   disp(['     Elevation Fresnel number: ',num2str(S_elev)]);
   disp(['Estimate of intensity maximum: ',num2str(1000*Int_max),' mm']);
end

return