%------------------------------------------------------------------------------
% Function    : resp1d(plotfig, weights, elem_pts, flagg, media, excitation, ..
%                      transducer, u0_opt, u1_opt, centers, mode);
%
% Description : plots the far-field array response from a 1d array transducer.
%               weights are half the transducer weights by symmetry.
%               if isempty(weigths) then user defined apodization or
%               uniform weighting is used in computation - depending
%               on excitation(5) parameter. 
%
%
% Language    : Matlab 4.2
% Written by  : Bjrnar Elgetun, Ifi-UiO
%
% Version no. : 1.0 BJE 07.07.95 first version
%               1.1 BJE 11.07.95 added -6 dB bandwidth on uniform plot
%               1.2 BJE 22.08.95 changed # elements detection
%               1.3 BJE 17.09.95 added SL-estimate due to weighting
%               1.4 BJE 02.11.95 check if weights exists, added transducer 
%                                in call
%               1.5 BJE 24.11.95 new input, included user defined apodization
%               1.6 SH  12.01.96 Accepts other windows specified by excitation(5) = 1..3
%                                Computes Integrated Sidelobe Ratio
%               1.7 SH  22.02.96 Removed symmetry contraint
%               1.8 SH  28.02.96 4096 plot points instead of 1024 to increase bw resolution
%                                ISL for unweighted also 
%               1.9 SH  29.02.96 Doubled u0_opt when searching for sidelobe peak
%                                (needed for quadratic opt. program)
%               2.0 SH  23.05.96 Speeded up calculation by structuring if's
%               2.1 SH  19.09.96 Improved bandwidth and ISL estimates by using a
%                                priori information
%               2.2 SH  02.10.96 Changed ISL to ISLR Integrated Sidelobe Ratio
%               2.3 AA  22.10.96 Changed lower limit for nsearch1 from 0 to 1
%                                Changed the overestimate of peak's width
%                                (nwidth) from +-5.0*L/D to +-1.2*L/D
%               2.4 SH  22.11.96 Finds and reports Peak value for unweighted
%               2.5 SH  27.11.96 Corrected peak to work on abs(W)
%                                Added output of average sidelobe level
%                                Peak and avg. counted from sidelobe region
%               2.6 SH  03.01.97 centers is now input => element directivity
%                                and curved arrays
%                                added check for nstart, nend
%               2.7 AA  22.10.96 Included call to findzero; finds 1. zero-point
%                                and redefines 6dB-points if required.
%                                Added thinning-pattern text to plot.
%                                Included debug-mode 
%                                  to mark 6dB points and mainlobe/sidelobe 
%                                  to mark minimized area and value to plot. 
%                                Adds test on weights. If weights for all
%                                elements, do nothing.                  
%                                Changed test on actice_right to ">=0" 
%                                (for odd number of elements)
%               2.8 SH  09.01.97 Changed thinning pattern text to analyze
%                                centers instead of elem_pts
%               2.9 AA  13.01.97 Changed lower limit of xpos to min(-1,-u1_opt)
%               3.0 AA  12.02.97 Changed test on active from ">0" to ">=0" to 
%                                make resp1d work with odd nu of el.   
%               3.1 AA  25.03.97 Included average in ylabel  
%               3.2 SH  04.06.97 Matlab 5: removed extra end, added round() 
%                                initialized bw etc, weighoption 
%               3.3 SH  12.06.97 Dotted line for expected mean value for
%                                thinned array
%               3.4 AA  19.06.97 Deleted lots of old commented code.
%                                Taking abs(W/uniW) at once. ==>
%                                    may delete a lot of abs()
%                                Added mode as input.
%                                mode > 0:1-way resp (org. version)
%                                mode < 0: 2-way response
%                                         -1: -6db BW
%               3.5 SH  24.07.97 Added abs for sum of user-defined weights
%               3.6 SH  02.09.97 Added dotted line for sqrt(Nlog(N)) - max
%                                value for peak sidelobe (Hendricks, Benke)
%               3.7 SH  22.09.98 Added more text output in report
%                                Added Chebyshev window. 
%                   SH  08.03.00 Test nstart, nend for outside of array
%
% Called by   : mslmain, wntmain, ffmain
% Calling     : vwctrl2, vinput, yinput, findzero
%------------------------------------------------------------------------------

function resp1d(plotfig, weights, elem_pts, flagg, media, excitation, ...
		transducer, u0_opt, u1_opt,centers, mode);


%	    
% To enable debugmode, comment in debug=1
%

debug=0;
%  debug=1;
  if debug==1
    fprintf('\nDebugmode enabled\n');
  end    
%
% initialization of transducer
%
  fprintf('Calculating array response ..\n');

  active=find(elem_pts(4,:) >= 0);	 % find active transducer elements
  active_right=find(elem_pts(1,:) >= 0 & elem_pts(4,:) > 0); 	% in right halfplane

  N_active=length(active); 		 % # active points
  N_active_el = N_active/transducer(3);  % # active elements      
  N_x=transducer(2); 			 % # elements x direction, full
  
  lambda = media(1,2)/excitation(1);

%
% find the centers coordinates corresponding to the elem_pts
%
  center_pts = zeros(size(elem_pts));
  for n = 1:size(centers,2);
    az = find(abs(elem_pts(4,:)) == abs(centers(4,n)));
    center_pts(:,az) = centers(:,n)*ones(size(az));
  end
%
% active elements' relative positions
%
  xpos=elem_pts(1,active)' ;
  zpos=elem_pts(3,active)' - center_pts(3,active)';
  
  uniweights=ones(N_active,1)./N_active; % uniform weight set
  phi_steer = excitation(4); 		 % steering angle in radians

%
% lambda normalization
%
  xpos  = xpos/lambda;
  zpos  = zpos/lambda;

  weightoption = 'x';
%
% read min plot level
%
  if (mode >= 0) 
    min_level=vinput('minimum response level for plot in dB',-100,-6,-50);
  else % 2-way response
    min_level=vinput('minimum response level for plot in dB',-200,-12,-100);
  end
  
  if (~isempty(weights) & length(weights)~=N_active)   % double weight vector
                                                      % if required
     weights = [fliplr(weights') weights']';
  end
%
% check arguments
%
%
% determine plotoption
%
  if (isempty(weights))
    weightoption='u'; 			        % user defined
    if (excitation(5) == 0) 			% no user defined apodization
      	plotoption='u';				% uniform only
      	weights=uniweights;
    elseif ((excitation(5) >=1 & excitation(5) <= 3) | excitation(5)==6)
      	plotoption = 'w';

        [apod_weight,apod_weight_az,apod_weight_el]=...
		apodrect(transducer, excitation, elem_pts);
        weights = apod_weight';
        weights=weights./sum(weights);	        % normalization

    elseif (excitation(5) == 4) % user-def apodization
      	plotoption = 'w';
      	global amp_ud;
      
      	if (isempty(amp_ud) | (size(amp_ud,2) ~= length(active)) )
          fprintf('\nError: amp_ud (user-definded weights) has incorrect format.\n');
          fprintf('       Perform new weighting or turn off user-defined apodization\n');     
          fprintf('       in beamforming submenu.\n\n');
          return;				% do nothing more !
      	end
      
      	weights=amp_ud(1,:)'; 			%  the weights
      	sumw = sum(abs(weights));                 % changed to add abs 24.07.97 SH
      	weights=weights./sumw;                    % normalization
    end
    
  else
%
% weights passed through input argument
%
  	if ~yinput('Append response plot for uniform weights',0)
      		plotoption='w';				% weighted only
  	else
      		plotoption='b';				% both
  	end
  end

%
% initialization of plot region
%
points=max(4*N_x,4096+1); 		% at least 4*1024 plot points
% points=511
  if (flagg(4) == 4) 			% plot region for u=sin(phi)

      u0=0;				% u-axis
      u0 = min(-1,-u1_opt);
      u1=max(1,u1_opt);			% [u0_opt,u1_opt] optimization interval
      h_u=(u1 - u0)/(points-1);
      P_u=[u0:h_u:u1]';			% plot points u coord
      P_zu = sqrt(1 - P_u.^2);          % plot points for z-coordinate
      ntop = 1 + (sin(phi_steer)-u0)/h_u;
      nwidth = (1.2*lambda/transducer(1))/h_u; % overestimate of peak's width
  
    else		% plot region for phi

      phi0=0;				% phi-axis
      phi0 = -pi/2;
      phi1=pi/2;
      h_phi=(phi1 - phi0)/(points-1);
      P_phi=[phi0:h_phi:phi1]';		% 'plot points phi coord
      ntop = 1 + (phi_steer-phi0)/h_phi;
      nwidth = (1.2*lambda/transducer(1))/h_phi; % overestimate of peak's width
  end

  
%
% calculating the array pattern on the plot points,
% find actual sidelobe level on array pattern and
% calculate actual -6dB bw on the array pattern
%
  if (flagg(4) == 4)		% u coordinates

    if (plotoption == 'b' | plotoption == 'w')      
%      W=exp(j*2*pi*( (P_u-sin(phi_steer))*xpos' ...
%                    +(P_zu-cos(phi_steer))*zpos' ))*weights;   
      W=abs(exp(j*2*pi*( (P_u-sin(phi_steer))*xpos' ...
                    +(P_zu-cos(phi_steer))*zpos' ))*weights);   
    end
    if (plotoption == 'b' | plotoption == 'u') 
%      uniW=exp(j*2*pi*( (P_u-sin(phi_steer))*xpos' ...
%                    +(P_zu-cos(phi_steer))*zpos' ))*uniweights;  
      uniW=abs(exp(j*2*pi*( (P_u-sin(phi_steer))*xpos' ...
                    +(P_zu-cos(phi_steer))*zpos' ))*uniweights);  
    end



  else       			% phi coordinates

    if (plotoption == 'b' | plotoption == 'w')      
%      W=exp(j*2*pi*( (sin(P_phi)-sin(phi_steer))*xpos' ...
%                    +(cos(P_phi)-cos(phi_steer))*zpos' ))*weights;   	% 'a vectorized shortform
      W=abs(exp(j*2*pi*( (sin(P_phi)-sin(phi_steer))*xpos' ...
                    +(cos(P_phi)-cos(phi_steer))*zpos' ))*weights);   	% 'a vectorized shortform

    end

    if (plotoption == 'b' | plotoption == 'u') 
%      uniW=exp(j*2*pi*( (sin(P_phi)-sin(phi_steer))*xpos' ...
%                    +(cos(P_phi)-cos(phi_steer))*zpos' ))*uniweights;  
      uniW=abs(exp(j*2*pi*( (sin(P_phi)-sin(phi_steer))*xpos' ...
                    +(cos(P_phi)-cos(phi_steer))*zpos' ))*uniweights);  
    end
  end

%
% Squaring of W/uniW in case of 2-way response
% 
  if (mode == -1) % 2-way response, -6db BW
    if (exist('W')==1), W = W.^2; end
    if (exist('uniW')==1), uniW = uniW.^2; end
  end
  
%
% Initialize search region around steered peak of beampattern
%
  nsearch1 = round(max(ntop - nwidth,1));      
  nsearch2 = round(min(ntop + nwidth,points));

%
% integrated sidelobes for weighted beam pattern
%
  if (plotoption == 'b' | plotoption == 'w')      
    
    n6dB = find(abs(W(nsearch1:nsearch2)) > 10^(-6/20)); % indices of peak
    n6dB_1 = min(n6dB) + nsearch1 - 1; 		     % beginning of peak
    n6dB_2 = max(n6dB)+ nsearch1 - 1;		     % end of peak
    [nstart,nend,n6dB_1,n6dB_2] = findzero(n6dB_1,n6dB_2,W,points);
    
    n6dB = n6dB_2 - n6dB_1 + 1; 		     % range of peak

%   2*(-6dB beamwidth) approx equal 3*(-3 dB beamwidth) as used by ESA 
    nstart = round(n6dB_1 - .5*n6dB); % findzero deals with "max(...)"
    nend   = round(n6dB_2 + .5*n6dB);
%    nstart = max(n6dB_1 - .5*n6dB,1); 
%    nend   = min(n6dB_2 + .5*n6dB,points);
    
    
    if (flagg(4) == 4)		                     % u coordinates
      bw=180/pi*(asin(n6dB_2*h_u+u0)-asin(n6dB_1*h_u+u0)+asin(h_u));
    else
      bw=180/pi*(n6dB-1)*h_phi;  
    end

    mainlobe = sum(abs(W(nstart:nend)).^2);
    sidelobe = sum(abs(W(1:nstart-1)).^2) + sum(abs(W(nend+1:length(W))).^2);
    ISLR = 10*log10(sidelobe/mainlobe);

    % average sidelobe level in dB
    plevel=20*log10(max(max(abs([W(1:nstart-1)' W(nend+1:length(W))']) )));
    % average sidelobe level in dB    
    average=20*log10(mean(mean(abs([W(1:nstart-1)' W(nend+1:length(W))']) )));
    
  end
  
%
% integrated sidelobes for unweighted beam pattern
%
  if (plotoption == 'b' | plotoption == 'u') 
    n6dB = find(abs(uniW(nsearch1:nsearch2)) > 10^(-6/20) ); % indices of peak
    n6dB_1 = min(n6dB)+ nsearch1 - 1; 			 % beginning of peak
    n6dB_2 = max(n6dB)+ nsearch1 - 1;			 % end of peak
    [nstart,nend,n6dB_1,n6dB_2] = findzero(n6dB_1,n6dB_2,uniW,points);
    n6dB = n6dB_2 - n6dB_1 + 1; 	                 % range of peak
    if (flagg(4) == 4)		                         % u coordinates
      unibw=180/pi*(asin(n6dB_2*h_u+u0)-asin(n6dB_1*h_u+u0)+asin(h_u));	
    else
      unibw=180/pi*(n6dB-1)*h_phi;  
    end
%
% Mainlobe is defined as 2*(-6dB beamwidth) 
% (approx equal 3*(-3 dB beamwidth) as used by ESA for Satellite Synthetic
% Aperture Radar image quality characterization)
% 
    nstart = round(n6dB_1 - .5*n6dB); % findzero deals with "max(...)"
    nend   = round(n6dB_2 + .5*n6dB);
%    nstart = max(n6dB_1 - .5*n6dB,1); 
%    nend   = min(n6dB_2 + .5*n6dB,points);
%
    nstart = max(nstart,2);
    nend = min(nend, length(uniW)-1);
    mainlobe = sum(abs(uniW(nstart:nend)).^2);
    sidelobe = sum(abs(uniW(1:nstart-1)).^2) + sum(abs(uniW(nend+1:length(uniW))).^2);
    ISLRun = 10*log10(sidelobe/mainlobe);

    % average sidelobe level in dB
    plevelu=20*log10(max(max(abs([uniW(1:nstart-1)' ...
	  uniW(nend+1:length(uniW))']) )));
    % average sidelobe level in dB
    averageu = 20*log10(mean(mean(abs([uniW(1:nstart-1)' uniW(nend+1:length(uniW))']) )));
    end

%
% init plot window
%
  vwctrl('dismiss',plotfig);			% clear old view controls
  vwctrl2('dismiss',plotfig);
  figure(plotfig);
  subplot(1,1,1); 				% clear old axes

  if (plotoption == 'b') 			% plot both responses
      axes2p=subplot(2,1,2);
      set(axes2p,'position',[.15 .1 .7 .25]);
      axes1p=subplot(2,1,1);			% split window
      set(axes1p,'position',[.15 .6 .7 .25]);
    else
      axes1p=subplot(1,1,1);			% one window
      set(axes1p,'position',[.15 .2 .7 .5]);
  end


%  
% Construct Thinning-pattern variable
%
  
  if (N_active_el ~= N_x)   % Thinning applied
    if (centers(4,1)<0)
      thin_pat = '0';
    else
      thin_pat = '1';
    end    
    for i=2:N_x
      if (centers(4,i)<0)
	thin_pat = sprintf('%s0',thin_pat);
      else
	thin_pat = sprintf('%s1',thin_pat);
      end
      if (rem(N_x,2)==0 & i==ceil(N_x/2))
	thin_pat = sprintf('%s c ',thin_pat);
      elseif (i==floor(N_x/2) | i==ceil(N_x/2))
	thin_pat = sprintf('%s ',thin_pat);
      end
    end

    thin_text = sprintf('Thinning pattern (%.1f%% thinned):', ...
	100*(1-N_active_el/N_x)); 
  end

%
% plot array response to uniform weights
%
  rad2deg = 180/pi;
  if (plotoption == 'b' | plotoption == 'u')
    
    if (flagg(4) == 4) 			% u coordinates
      
      plot( P_u,20*log10(abs(uniW)),'r');
      if (N_active < N_x)
% plot dotted line for average for random array	
% a line for peak
	hold on; 
	if (mode >= 0)
	  plot(P_u, -10*log10(N_active)*ones(size(P_u)),':', ...
	      P_u,-10*log10(sqrt(N_active*log(N_active)))*ones(size(P_u)),':'); 
	else
	  plot(P_u, -20*log10(N_active)*ones(size(P_u)),':', ...
	      P_u,-20*log10(sqrt(N_active*log(N_active)))*ones(size(P_u)),':'); 
	end	  
	hold off;
      end
      axis([u0 u1 min_level 0]);
      xlabel('sin(phi)');
%      ylabel(['[dB], ISLR ',num2str(ISLRun),' dB']);
      ylabel(['[dB], ISLR ',num2str(ISLRun), ', Avg ', num2str(averageu)]);
      
      if (N_active_el~= N_x)   % Thinning applied
	text(-u1,min_level*9/7.5,thin_text)
	if (length(thin_pat)<50)
	  text(-u1,min_level*9.6/7.5,thin_pat);
	else
	  text(-u1,min_level*9.6/7.5,thin_pat(1:floor(N_x/2)+rem(N_x,2)*2));
	  h = text(min(-N_x/100,-.75),min_level*10.1/7.5, ...
	      thin_pat(floor(N_x+3-N_x/2):N_x+3-rem(N_x,2)));
	end	    
      end
      
    else					% phi coordinates
      
      plot( rad2deg*P_phi, 20*log10(abs(uniW)),'r');
      if (N_active < N_x)
% plot dotted line for average for random array	
	hold on; 
	if (mode >= 0)
	  plot(rad2deg*P_phi, -10*log10(N_active)*ones(size(uniW)),':', ...
	      rad2deg*P_phi,-10*log10(sqrt(N_active*log(N_active)))*ones(size(P_phi)),':'); 
	else
	  plot(rad2deg*P_phi, -20*log10(N_active)*ones(size(uniW)),':', ...
	      rad2deg*P_phi,-20*log10(sqrt(N_active*log(N_active)))*ones(size(P_phi)),':'); 
	end	  
	hold off;
      end
      axis([rad2deg*phi0 rad2deg*phi1 min_level 0]);
      xlabel('phi');
%      ylabel(['[dB], ISLR ',num2str(ISLRun),' dB']);
      ylabel(['[dB], ISLR ',num2str(ISLRun),', Avg ', num2str(averageu)]);
      
      if (N_active_el~= N_x)   % Thinning applied
	text(-90,min_level*9/7.5,thin_text)
	if (length(thin_pat)<50)
	  text(-90,min_level*9.6/7.5,thin_pat);
	else
	  text(-90,min_level*9.6/7.5,thin_pat(1:floor(N_x/2)+rem(N_x,2)*2));
	  text(-90*min(N_x/100,.75),min_level*10.1/7.5, ...
	      thin_pat(floor(N_x+3-N_x/2):N_x+3-rem(N_x,2)));
	end	   
      end
	  
    end
	
%      T=sprintf('Response to uniform weighted array   -6 dB BW: %4.2f [deg]',unibw);
    if (mode >= 0)
      T=sprintf(['1-way response to unweighted array   Peak: %4.1f [dB]',...
	    '   -6 dB BW: %4.2f [deg]'], plevelu, unibw);
    else
      T=sprintf(['2-way response to unweighted array   Peak: %4.1f [dB]',...
	    '   -6 dB BW: %4.2f [deg]'], plevelu, unibw);
    end
    title(T);
    
    grid;
    if (debug == 1 & N_active_el~= N_x)
      hold on;
      if (flagg(4) == 4) 
	n0=ceil((u0_opt+u1)/h_u) + 1;  % index where optimized region starts
	fill([u1 P_u(n0) P_u(n0) u1_opt u1_opt],[0 0 min_level min_level 0],'b');
	fill([P_u(n0); [P_u(n0:points)]; P_u(points); P_u(n0)], ...
	    [min_level; [20*log10(abs(uniW(n0:points)))]; min_level ...
	      ;min_level],'y');  
	% mainlobe - sidelobe
	plot([nstart nstart]*h_u-1,[0 min_level],'-.y');
	plot([nend nend]*h_u-1,[0 min_level],'-.y');
	% 6dB point
	plot([n6dB_1 n6dB_1]*h_u-1,[0 min_level],'--y');
	plot([n6dB_2 n6dB_2]*h_u-1,[0 min_level],'--y');
      else
	n = pi*((asin(u0_opt)+1)/2)/h_phi;
	fill([90 P_phi(n)*rad2deg P_phi(n)*rad2deg 90 90], ...
	    [0 0 min_level min_level 0],'b');
	fill([P_phi(n); [P_phi(n:points)]; P_phi(points); ...
	  P_phi(n)]*rad2deg, [min_level; [20*log10(abs(uniW(n:points)))]; ...
	  min_level; min_level],'y');  
	
	int_s_l = 2* sum(abs(uniW(n:points)).^2);
	% mainlobe - sidelobe
	plot([nstart nstart]*rad2deg*h_phi-90,[0 min_level],'-.y');
	plot([nend nend]*rad2deg*h_phi-90,[0 min_level],'-.y');
	% 6dB point
	plot([n6dB_1 n6dB_1]*rad2deg*h_phi-90,[0 min_level],'--y');
	plot([n6dB_2 n6dB_2]*rad2deg*h_phi-90,[0 min_level],'--y');
      end
      
      hold off;
    end

  end

%
% plot array pattern to optimized weights and print statistics
%
  if (plotoption == 'b' | plotoption == 'w')
    
    if (plotoption == 'b')		% second plot
      axes(axes2p);
    end
    
    if (flagg(4)==4) 			% u coordinates
      
      plot( P_u,20*log10(abs(W)),'r');
      if (N_active < N_x)
% plot dotted line for average for random array	
	hold on; 
	if (mode >= 0)
	  plot(P_u, -10*log10(N_active)*ones(size(P_u)),':', ...
	      P_u,-10*log10(sqrt(N_active*log(N_active)))*ones(size(P_u)),':'); 
	else
	  plot(P_u, -20*log10(N_active)*ones(size(P_u)),':', ...
	      P_u,-20*log10(sqrt(N_active*log(N_active)))*ones(size(P_u)),':'); 
	end	  
	hold off;
      end
      axis([u0 u1 min_level 0]);
      xlabel('sin(phi)');
%      ylabel(['[dB], ISLR ',num2str(ISLR),' dB']);
      ylabel(['[dB], ISLR ',num2str(ISLR),', Avg ', num2str(average)]);
      if (N_active_el~= N_x)   % Thinning applied
	text(-u1,min_level*9/7.5,thin_text);
	if (length(thin_pat)<50)
	  text(-u1,min_level*9.6/7.5,thin_pat);
	else
	  text(-u1,min_level*9.6/7.5,thin_pat(1:floor(N_x/2)+rem(N_x,2)*2));
	  text(min(-N_x/100,-.75),min_level*10.1/7.5, ...
	      thin_pat(floor(N_x+3-N_x/2):N_x+3-rem(N_x,2)));
	end	    	  
      end
    else				% phi coordinates
      
      plot( rad2deg*P_phi,20*log10(abs(W)),'r');
      if (N_active < N_x)
% plot dotted line for average for random array	
	hold on; 
	if (mode >= 0)
	  plot(rad2deg*P_phi, -10*log10(N_active)*ones(size(P_phi)),':', ...
	      rad2deg*P_phi,-10*log10(sqrt(N_active*log(N_active)))*ones(size(P_phi)),':'); 
	else
	  plot(rad2deg*P_phi, -20*log10(N_active)*ones(size(P_phi)),':', ...
	      rad2deg*P_phi,-20*log10(sqrt(N_active*log(N_active)))*ones(size(P_phi)),':'); 
	end	  
	hold off;
      end
      axis([rad2deg*phi0 rad2deg*phi1 min_level 0]);
      xlabel('phi');
      ylabel(['[dB], ISLR ',num2str(ISLR),' dB']);
      ylabel(['[dB], ISLR ',num2str(ISLR),', Avg ', num2str(average)]);
      
      if (N_active_el~= N_x)   % Thinning applied
	text(-90,min_level*9/7.5,thin_text);
	if (length(thin_pat)<50)
	  text(-90,min_level*9.6/7.5,thin_pat);
	else
	  text(-90,min_level*9.6/7.5,thin_pat(1:floor(N_x/2)+rem(N_x,2)*2));
	  text(-90*min(N_x/100,.75),min_level*10.1/7.5, ...
	      thin_pat(floor(N_x+3-N_x/2):N_x+3-rem(N_x,2)));
	end	    
      end
    end
%
% add text and grid
%
    if (weightoption == 'u')
      if (mode >= 0)
	T=sprintf('1-way response to user-defined weights',  ...
	    '  -6  dB BW:  4.2f [deg]',bw); 
      else
	T=sprintf('2-way response to user-defined weights',  ...
	    '  -6  dB BW:  4.2f [deg]',bw); 
      end	
    else
      if (mode >= 0)
	T=sprintf(['1-way response optimized from: %4.2f [deg]', ...
	      '   Peak:%4.1f [dB]   -6 dB BW: %4.2f [deg]'], ...
	    asin(u0_opt)*180/pi, plevel, bw);  
      else
	T=sprintf(['2-way response optimized from: %4.2f [deg]', ...
	      '   Peak:%4.1f [dB]   -6 dB BW: %4.2f [deg]'], ...
	    asin(u0_opt)*180/pi, plevel, bw);  
      end	
    end

    title(T);
    grid;
    if (debug == 1 & N_active_el~= N_x)
      hold on;
      if (flagg(4)==4)      
	n0=ceil((u0_opt+u1)/h_u) + 1;
	n1=ceil((u1_opt+u1)/h_u) + 1;
	fill([u1_opt P_u(n0) P_u(n0) u1_opt u1_opt],[0 0 ...
	      min_level min_level 0],'b'); 
	fill([P_u(n0); [P_u(n0:n1)]; P_u(n1); P_u(n0)], ...
	    [min_level; [20*log10(abs(W(n0:n1)))]; min_level ...
	      ;min_level],'y'); 
	int_s_l = 2* sum(abs(W(n0:points)).^2);
	% mainlobe - sidelobe
	plot([nstart nstart]*h_u-1,[0 min_level],'-.y');
	plot([nend nend]*h_u-1,[0 min_level],'-.y');
	% 6dB point
	plot([n6dB_1 n6dB_1]*h_u-1,[0 min_level],'--y');
	plot([n6dB_2 n6dB_2]*h_u-1,[0 min_level],'--y');
      else
	n = pi*((asin(u0_opt)+1)/2)/h_phi;
	fill([90 P_phi(n)*rad2deg P_phi(n)*rad2deg 90 90], ...
	    [0 0 min_level min_level 0],'b');
	fill([P_phi(n); [P_phi(n:points)]; P_phi(points); ...
	  P_phi(n)]*rad2deg, [min_level; [20*log10(abs(W(n:points)))]; ...
	  min_level; min_level],'y');  
	int_s_l = 2* sum(abs(W(n:points)).^2);
	% mainlobe - sidelobe
	plot([nstart nstart]*rad2deg*h_phi-90,[0 min_level],'-.y');
	plot([nend nend]*rad2deg*h_phi-90,[0 min_level],'-.y');
	% 6dB point
	plot([n6dB_1 n6dB_1]*rad2deg*h_phi-90,[0 min_level],'--y');
	plot([n6dB_2 n6dB_2]*rad2deg*h_phi-90,[0 min_level],'--y');
      end
    end
    hold off;
  end
  
%
% calculate estimated SL-due to aliasing
%
      w_sum=2*sum(weights);		% only half the symmetric sum-normalized weights
      w_sumsqr=2*norm(weights)^2;

      b=w_sumsqr/w_sum;			% ENBW (not normalized)

  if (plotoption =='u')
    bw = unibw;
    plevel = plevelu;
    average = averageu;
  end
%
% print statistics
%
      fprintf('\n\n---------------------------------------------------------\n');
      fprintf('Response calculation statistics:\n\n');
      fprintf('Actual Beamwidth (-6 dB) :            %10.2f [deg], delta: %6.4f\n',bw,180/points);
      fprintf('Peak sidelobe level:                  %10.2f [dB]\n',plevel);
      fprintf('Average sidelobe level:               %10.2f [dB]\n',average);
  if (plotoption == 'b') 		% plot both responses
	   fprintf('Average sidelobe level (unweighted):  %10.2f [dB]\n',averageu);      
  end   
     fprintf('---------------------------------------------------------\n');
     
     bwtheory = (180/pi)*asin(1.21*lambda/transducer(1));
      fprintf('Beamwidth (-6 dB) of full, unweighted %10.2f [deg]\n',bwtheory );
  p=1-N_active_el/N_x;      		% thinning
  
  if p>0
    fprintf('Theoretical peak: 	              %10.2f [dB]\n',-10*log10(sqrt(N_active*log(N_active))));
    fprintf('Theoretical average: 	              %10.2f [dB]\n',-10*log10(N_active));

  end
  fprintf('---------------------------------------------------------\n');
  if strcmp(weightoption,'u')
      fprintf('\nFinished!\n\n')
  end


