function [nx,ny,nz] = normal(x,y,z,htype,zhcross)
%------------------------------------------------------------------------------
% Routine         : normal.m
% Description     : Finds the normal of a surface at point (x,y,z)
%                   htype=[functype,   par1,  par2,  par3, par4      par5]
%                   polynom:   [  1  ,   a,     b,    c,      d     dummy]
%                   elliptic:  [  2  ,   rx,   ry,    rz,   sgnfoc,   dz]
%                   warped:    [  6  ,   sn,   xt,    yt,        a,  dummy ]
%                   simple:    [  7  ,   lx,   ly,    nx,       ny,  a ]
%
%                   polynom:  h=ax+bx^2+cy+dy^2
%                   elliptic: rz=radius in z-direction
%                             rx=radius in x-direction
%                             ry=radius in y-direction
%                             sgnfoc=-1 when focus is neg. and 1 when pos.
%                             dz=extra length in z-direction 
%                             h=rz*(1-sqrt(1-(x/rx).^2-(y/ry).^2))*sgnfoc   
%			      when (x/rx)^2 + (y/ry)^2<=1
%                             h=(rz+dz)*sgnfoc   when (x/rx)^2 + (y/ry)^2>1
%                   warped:   sn = surface mesh no. (in glob. var. smeshes)
%                             xt,yt = translate position of centre of mesh 
%                                     in xy plane
%                             a =  sclaing of  amplitude of h
%                   simple:   lx,ly = dimensions of warped region 
%                             nx,ny = number of sine/cosine periods 
%                             a     = amplitude 
%
%		    
%
% Language        : Matlab 4.1
% Written by      : Lars Odegaard & Frode Teigen, IBT
% Version no.     : 3.0 LO/FT 13.10.93 First Version
%                   3.1 EH    21.11.95 Added warped surface.
%                   3.2 EH    22.12.95 changed parameters for warped surface
%                   3.3 EH    19.02.95 Added simple warped surface. (no. 7)
%
% Called by       : snellaw
% Routines called : simpnrm
%----------------------------------------------------------------------------- 
global smeshes;

if htype(1)==1 

% polynom
% z=a*x+b*x.^2+c*y+d*y.^2

  a=htype(2);
  b=htype(3);
  c=htype(4);
  d=htype(5);
  nx = a*ones(size(x)) + 2*b*x;
  ny = c*ones(size(x)) + 2*d*y;
  nz = -1*ones(size(x));

elseif htype(1)==2

% elliptic
% h=rz*(1-sqrt(1-(x/rx)^2-(y/ry)^2))*sgnfoc when (x/rx)^2 + (y/ry)^2<=1
% h=(rz+dz)*sgnfoc                          when (x/rx)^2 + (y/ry)^2>1

  [m,n]=size(x);
  xx=x(:);
  yy=y(:);
  rz=htype(4);
  ry=htype(3);
  rx=htype(2);
  sgnfoc=htype(5);
  dz=htype(6);
  nxx = zeros(size(xx));
  nyy = zeros(size(xx));
  nzz = -1*ones(size(xx));
  zz = z(:);
  ind1 = find(((xx/rx).^2 + (yy/ry).^2)<=1);
  nxx(ind1) = sgnfoc*2*xx(ind1)/rx^2;
  nyy(ind1) = sgnfoc*2*yy(ind1)/ry^2;
  nzz(ind1) = -2*(1-sgnfoc*(zz(ind1)-zhcross)/rz)/rz;
   
% The following is reminiscences of the 2D code for boundary 
% partially converted to 3D
% This should be implemented before using an offset dz~=0
%  if dz~=0,
    % To avoid h diskont.
%    dx=1e-5;
%    dy=1e-5;
%    y0 = [      -ry-dz,       -ry,        ry,          ry+dz];
%    x0=[        -rx-dz,       -rx,        rx,          rx+dz];
%    h0=[(rz+dz)*sgnfoc, rz*sgnfoc, rz*sgnfoc, (rz+dz)*sgnfoc];
 %   [ind2=find(((((x-dx)/rx).^2 + ((y-dy)/ry).^2)<=1)&(((x/rx).^2 + (y/ry).^2)>=1));
%    h(ind2)=interp1(x0,h0,x(ind2));
%  end
%
  nx=reshape(nxx,m,n);
  ny=reshape(nyy,m,n);
  nz=reshape(nzz,m,n);

elseif htype(1)==6 

% warped 
  mstart = 10;
  sn = htype(2);
  xt = htype(3);
  yt = htype(4); 
  a = htype(5);
  l = smeshes(sn,4);
  x0 = smeshes(sn,2);
  y0 = smeshes(sn,3);
  nmesh = smeshes(sn,1);
  surf = a*reshape(smeshes(sn,mstart:(mstart-1+nmesh^2)),nmesh,nmesh);
  xm =  (0:(nmesh-1))/(nmesh-1) - 0.5;
  xm =  l*xm + x0 + xt;
  ym =  (0:(nmesh-1))/(nmesh-1) - 0.5;
  ym =  l*ym  + y0 + yt;
  [X,Y] = meshgrid(xm,ym);
  [nx0,ny0,nz0] = surfnorm(X',Y',surf'); 

  nx = interp2(xm,ym,nx0,x,y);
  ny = interp2(xm,ym,ny0,x,y);
  nz = interp2(xm,ym,nz0,x,y);

  index = find(isnan(nx) | isnan(ny) | isnan(nz));
  if ~isempty(index)
     nx(index) = zeros(size(index));  
     ny(index) = zeros(size(index));  
     nz(index) = -1*ones(size(index));  
  end

elseif htype(1)==7
  [nx,ny,nz] = simpnrm(htype(2:6),x,y);
   nx = -nx; 
   ny = -ny;
   nz = -nz;

end


% Normalizing Normal Vector
%--------------------------

len = sqrt(nx.^2 + ny.^2 + nz.^2);
nx = nx./len;
ny = ny./len;
nz = nz./len;












