%----------------------------------------------------------------
% Routine     : pecomp.m 
%
% Description : Computes the time delay to all (Xo,Yo,Zo).        
%               It then adds up the contributions from each
%               array point specified to each field point (X,Y,Z)   
%
% Language    : Matlab V4.0
% Written by  : Vingmed sound A/S, Trond Kleveland 
%
% Version no. : 1.0 TKL 14.09.93, First version
%             : 1.1 TKL 22.09.93, Corrected CurrElTd, removed analysis
%             : 1.2 TKL 23.09.93, Added time-analysis
%             : 1.3 TKL 12.11.93, Calculating exact delays from focus.m
%                                 (phase center instead of elem.center)
%             : 1.4 TKL 22.11.93, Added save to file
%             : 1.5 TKL 24.11.93, Added yz-plane to analysis
%             : 1.6 TKL 29.11.93, Speeded up CW calculations
%                                 corrected r_attenuation
%             : 1.7  SH 30.11.93, Removed option for attenuation
%				  Added stopwatch
%             : 1.8 TKL 30.11.93, No checking left for attenuation
%             : 1.9  SH 10.09.93, Changed attenuation for annular arrays
%             : 2.0 TKL 13.12.93, Added xy-plane to analysis
%	      : 2.1  SH 17.01.94, Added c_foc 
%             : 2.2  FT 17.02.94, c_foc = excitation(14)
%	      : 2.3 BJE 12.10.94, prepared for movie
%	      : 2.4 BJE 21.11.94, Added max_resp, min_resp
%             : 2.5 SH   4. 1.95  Dynamic focusing and aperture implemented
%             : 2.6 SH  19.01.95  CPU-time and flop counter
%             : 2.7 SH  10.04.95  Corrected display of f/stop in dynamic focus
%             : 2.8 SH  12.04.95  Excitation(2:4) now in polar coordinates
%             : 2.9 BJE 23.04.95  Removed ConfigOrigHandle, PlotfigOrigHandle
%                                 from function call
%             : 3.0 SH  11.05.95  Corrected input to focus() when dynamic focus
%               3.1 SH  16.05.95  Checking for valid input (0 pulse length,
%                                 0 obs. time, 0 obs/mm) 
%             : 3.2 KE   2.05.96  Added Mex capability and 3D observations
%               3.3 SH  05.06.96  Added centers as input to pe and to focus
%               3.4 SH  13.08.96  Split out pec.m from pecomp.m in order to
%                                 make mex file invisible for user
%               3.5 SH  04.09.96  Initialized F_des   
%               3.6 SH  29.05.97  Matlab 5: initialized delim to remove warning
%               3.7 SH   7.04.98  Added attenuation parameters
%                   SH  15.03.01  Removed flop counter - not supported in Matlab 6.
%
% Called by   : usimcnf
% Calling     : cleanscr, xyz2rhp, apod, focus 
%----------------------------------------------------------------

function [resp,max_resp,min_resp]=pecomp(excitation,transducer,media, ...
         observasjon,option,flagg,x,y,z,t,elem_pts,centers,config, ...
         plotfig,usr_resdir)
  

%
% INITIALIZATION
%
  clc
% cleanscr(config,ConfigOrigHandle)
% cleanscr(plotfig,PlotfigOrigHandle)

  max_resp=[0 0]; min_resp=[0 0];	% real and imaginary part

%
% EXCITATION
%
  osc = excitation(9);        % transmitted pulse length (#osc)
  Weight = excitation(10);    % pulse weighing (Rect=0,Cos=1)
  f = excitation(1);          % transmit frequency
% 
% excitation(2:4) now come in spherical coordinates - no need to convert
% SH 12.04.95
%
  r = excitation(2);
  theta = excitation(3);
  phi = excitation(4);

%
% TRANSDUCER
%
  ElPt=size(elem_pts);
  Ne_az=transducer(2);
  Np_az=transducer(3);
  Me_el=transducer(6);
  Mp_el=transducer(7);
  d=transducer(1);
  N=transducer(2);
%
  F_des = -1;
  delim = '/';
%
% OPTION
%
  if all(option == '2xz')          
      disp('  ')
      disp('Running Distance-based analysis in the xz-plane')
      disp('  ')
    elseif all(option == '2yz')
      disp('  ')
      disp('Running Distance-based analysis in the yz-plane')
      disp('  ')
    elseif all(option == '2xy') 
      disp('  ')
      disp('Running Distance-based analysis in the xy-plane')
      disp('  ')
    elseif all(option == '2xt') 
      disp('  ')
      disp('Running Time-based analysis in the xt-plane')
      disp('  ')
    elseif (option(1) == 'm')
      disp('  ')
      disp('Running Time-based analysis for movie')
      disp('  ')
% ---------------------------------------------------
% Added for 3D simulations by KE
    elseif (option(1) == '3')
      disp('  ')
      disp('Running Distance based analysis for xyz-volume')
      disp('  ')
    elseif (option(1) == 'f')
      disp('  ')
      disp('Running Time-based analysis for 3D movie')
      disp('  ')
% ---------------------------------------------------
    else
      disp('  ')
      disp('ERROR in specifications - please recheck that :')
      disp('  ')
      disp('  Observation PLANE is selected')
      disp('    - X or Y is 1.st axis')
      disp('    - Z or T is 2.nd axis')
      disp('  ')
      return 
  end
%
% check for legal input
%
  if (osc == 0) 
    disp('Pulse length is zero, for CW use Inf instead!')
    return
  end
  if (observasjon(4) == 0)
    disp('Observation time cannot be 0!')
    return
  end
  if (observasjon(15) == 0)
    disp('Number of observation points per mm cannot be 0!')
    return
  end  
%  
% MEDIUM
%
  c = media(1,2);
  c_foc = excitation(14); % Speed of sound used when calculating 
%                           focusing delays.
  alfa = media(1,9);     % attenuation parameter
  beta = media(1,10);     % attenuation parameter

%  
% COMPUTED PARAMETERS
%
  PulseLength  = osc/f;
  omega = 2*pi*f; 
%
% APODIZATION
% for annular arrays, an uneven distribution of points per element is
% automatically compensated for
%
  [elem_apo]=apod(transducer, excitation, elem_pts, flagg); 
  Apodize=ElPt(2)*elem_apo/sum(elem_apo);
  
  disp('  ')
  
%
% size of field
%
  FieldSize = size(x);
  FieldLength = FieldSize(1)*FieldSize(2);
%
% make up a vector of analysis times 
%
%--------------------------
% Check also for 3D by KE
  if option(1) == 'm' | option(1)=='f'	% movie
% -------------------------
      delta_t=1/(1e3*c*observasjon(15)); % from c and #obs
      TimeStepVector = [observasjon(4):delta_t:observasjon(8)];      
      TLen = length(TimeStepVector);
%
%   determine computer-dependent path separator for storing movie files
%
    cpu = computer;
    if (cpu(1:2) == 'PC'),
        delim = '\';
    else
        delim = '/';
    end
  else
      TimeStepVector = [observasjon(4)];
      TLen = 1;
  end
%
% FN directly from range, SH 12.04.95
%
  FN = r/transducer(1);   
% ---------------------------------
% Check for 3d by KE/corrcted from | to & 05.06.96 SH
  if (flagg(1) == 1) | ((option(1) ~= 'm' ) & (option(1) ~='f')), % fixed focus or not movie 
% ---------------------------------
%
% CALCULATE EXACT STEERING AND FOCUSING DELAYS 
%
% New format for disp string, SH 12.04.95
%
     disp([' Fixed focus to ',num2str(1000*r),' mm, (',num2str(phi),  ...
          ', ',num2str(theta),') deg, f-number ',num2str(FN)]);
     disp(' ');  
     [SteerFocusDelay,ctr] = focus(excitation,c_foc,transducer,elem_pts,centers,flagg);
     disp(' ');
   else
     disp(['Continuous dynamic focus from ',num2str(1000*c*TimeStepVector(1)),' to ', ...
	   num2str(1000*c*TimeStepVector(TLen)),' mm']);
     F_des = input('Enter f-number for dynamic aperture control: (<= 0: fixed aperture): ');
     disp(' ');
     if (excitation(5) ~= 0) & (excitation(5) ~= 5), 
       disp(' ');
       disp('Apodization and dynamic aperture is unpredictable!')
       disp(' ');
     end
     if (flagg(2) == 3),
       disp(' ');
       disp('Dynamic aperture and annular array is unpredictable!')
       disp(' ');
     end
   end  
%
% start stopwatch, cputimem and flop counters
%
  tic
  cpu1 = cputime;
%%%  flop1 =flops; % no longer supported in matlab 6
%
% LOOP OVER TIME REFERENCES
%
% ------------------------------------
% Added for Mex Programming by KE

  if (flagg(2)==1),
     trans_arr_val=1/(Np_az*Mp_el);
  else
     trans_arr_val=Ne_az/Np_az;
  end
%
% added alfa, beta 7.4.98 SH
%
  
  val=[r c PulseLength TLen omega Weight trans_arr_val F_des alfa beta];

  [resp,max_resp,min_resp]=pec(x,y,z,t,elem_pts,Apodize,excitation,SteerFocusDelay,...
                             TimeStepVector,transducer,flagg,option,usr_resdir,delim,val,centers);

% -------------------------------------------


%
% stop stopwatch & print elapsed time
%
  cpu2 = cputime;
  %%% flop2 = flops; % no longer supported in matlan6
  fprintf('\n\n       CPU time: %8.2f seconds\n',cpu2-cpu1);
  fprintf('   Elapsed time: %8.2f seconds\n',toc);
  %%fprintf('Number of flops: %8.2f, flops/sec: %8.2f\n\n', ...
  %%         flop2-flop1,(flop2-flop1)/(cpu2-cpu1));
%
% print end-message 
%
  fprintf('Computation is finished.\n\n>> ');
 

